

/* Max relative rms error is <0.5% for deg-1 and <2% for deg-2 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <x2geo.h>

static FILE *cfile;

static int indx
(
   int n,
   int m
)
{
   int i;

   if (m == 0) 
   {
      i = (n - 1) * (n + 1) + 1;
      return(i-1);
   }
   if (m >= 0)
   {
      i = (n - 1) * (n + 1) + m * 2;
   }
   else
   {
      i = (n - 1) * (n + 1) - m * 2 + 1;
   }
   return(i-1);
}  
/*indx*/

static int i2m
(
   int i
)
{
   int m;

   m = (i+1) / 2;
   if ((i & 1) == 0)
   {
      m = -m;
   }
   return(m);
}
/*i2m*/

int gsm2geo_init
(
   char   *fname, 
   int    *deg_ext,
   int    *deg_day,
   int    *deg_ann,
   int    maxcoeff,
   double *coeff
)
{
   int i,igsm,igeo,iday,iann,ncoeff;
   int n,mgeo,mgsm,i1,i2;
   int ncoeff_ext,ncoeff_day,ncoeff_ann,ie0,ie1,id,in0,im0,in1,im1;

   cfile = fopen(fname, "r");
   if (cfile == NULL) 
   {
      printf
          (
             "file %s could not be opened\n", fname
          ); 
      exit(11);
   }
   fscanf(cfile,"%*[^\n]");
   getc(cfile);
   fscanf(cfile,"%d%d%d%*[^\n]",deg_ext,deg_day,deg_ann);
   
   ncoeff_ext = *deg_ext*(*deg_ext+2);
   ncoeff_day = *deg_day*2+1;
   ncoeff_ann = *deg_ann*2+1;
   ncoeff     = ncoeff_ext * ncoeff_ext * ncoeff_day * ncoeff_ann;
   
   if (ncoeff > maxcoeff) 
   {
      printf
          (
             "array size for coefficients is too small!"
          ); 
      exit(12);
   }
   
   getc(cfile);
   fscanf(cfile,"%*[^\n]");
   getc(cfile);
   fscanf(cfile,"%*[^\n]");
   getc(cfile);
   
   i = 0;

   for (n = 1; n <= (*deg_ext); n++)
   {
      for (i1 = 0; i1 <= (2*n); i1++)
      {
         mgsm = i2m(i1);
         igsm = indx(n,mgsm);
         
         for (i2 = 0; i2 <= (2*n); i2++)
         {
            mgeo = i2m(i2);
            igeo = indx(n,mgeo);
            
            for (iday = 0; iday < ncoeff_day; iday++)
            {
               fscanf
                   (
                      cfile,
                      "%d%d%d%d",
                      &in0,
                      &im0,
                      &im1,
                      &id
                   );
               /*
               printf
                  (
                     "gsm read: n %d m %d m %d iday %d\n",
                     in0,im0,im1,id
                  ); 
               */

               ie0 = indx(in0,im0);
               ie1 = indx(in0,im1);

               if 
                   (
                      (ie0 != igsm)          || 
                      (ie1 != igeo)          || 
                      (id  != iday-*deg_day) 
                   )
               {
                  printf
                      (
                         "gsm2geo loop: n %d m %d n %d m %d iday %d\n",
                         n, mgsm, n, mgeo, id
                      );
                  printf
                      (
                         "gsm2geo read: n %d m %d n %d m %d iday %d\n",
                         in0, im0, in1, im1, id
                      );
                  printf
                      (
                         "gsm2geo: input file suspicious: %d <> %d or %d <> %d or %d <> %d\n",
                         ie0, igsm, ie1, igeo, id, iday-*deg_day
                      );
                  exit(13);
               }
               
               for(iann = 0; iann < ncoeff_ann; iann++)
               {
                  fscanf(cfile,"%lg",&coeff[i]);
                  /*
                  printf
                     (
                        "%12.5f\n",
                        coeff[i]
                     ); 
                  */
                  i++;
               }

               fscanf(cfile,"%*[^\n]");
               getc(cfile);

            }/*iday*/
         }/*i2*/
      }/*i1*/
   }/*deg_ext*/
      
   fclose(cfile);
   return(ncoeff);
}

int gsm2geo
(
   double fday,
   int    deg_want,                               /* chosen */
   int    deg_ext,                                /* given  */
   int    deg_day,
   int    deg_ann,
   double coeff[], 
   double gsmcoeff[],
   double geocoeff[]
)
{
  int    ncoeff_want = deg_want*(deg_want+2);
  int    ncoeff_ext  = deg_ext*(deg_ext+2);
  int    ncoeff_day  = deg_day*2 + 1;
  int    ncoeff_ann  = deg_ann*2 + 1;
  int    ncoeff      = ncoeff_ext * ncoeff_ext * ncoeff_day * ncoeff_ann;
  int    igeo,igsm,iday,md,ma,ic;
  int    n,mgeo,mgsm,i1,i2;
  double harm_a,harm_d;

  for (igeo = 0; igeo < ncoeff_want; igeo++) 
  {
     geocoeff[igeo] = 0.0;
  }

  ic = 0;
  for (n = 1; n <= deg_want; n++)
  {
     for (i1 = 0; i1 <= 2*n; i1++)
     {
        mgsm = i2m(i1);
        igsm = indx(n,mgsm);
        
        for (i2 = 0; i2 <= 2*n; i2++)
        {
           mgeo = i2m(i2);
           igeo = indx(n,mgeo);
           
           for (md = -deg_day; md <= deg_day; md++) 
           {
              if (md == 0) harm_d = 1.0;
              if (md > 0)  harm_d = SQRT2 * cos(md * WD * fday);
              if (md < 0)  harm_d = SQRT2 * sin(md * WD * fday);
              
              for (ma = -deg_ann; ma <= deg_ann; ma++) 
              {
                 if (ma == 0) harm_a = 1.0;
                 if (ma > 0)  harm_a = SQRT2 * cos(ma * WA * fday);
                 if (ma < 0)  harm_a = SQRT2 * sin(ma * WA * fday);
                 
                 geocoeff[igeo] += gsmcoeff[igsm] * harm_d    * 
                                   harm_a         * coeff[ic];
                 ic++;
                 /*
                 printf
                    (
                       "%d %d %d %d %d %12.5f\n",
                       igsm,igeo,md,ma,ic,coeff[ic]
                    ); 
                 */
              }/*for ma*/
           }/*for md*/
        }/* for mgeo */
     }/* for mgsm */
  }/*deg_want*/

  return(0);
} 
/*gsm2geo*/

int gsm2geo_green
(
   double         fday,                       /* chosen */
   int            deg_want, 
   int            deg_ext,                    /* given */
   int            deg_day,
   int            deg_ann,
   double         coeff[], 
   tgsm2geo_green green_day_r,
   tgsm2geo_green green_day_i,
   tgsm2geo_green green_ann_r,
   tgsm2geo_green green_ann_i
)
{
   int    ncoeff_want = deg_want*(deg_want+2);
   int    ncoeff_ext  = deg_ext*(deg_ext+2);
   int    ncoeff_day  = deg_day*2+1;
   int    ncoeff_ann  = deg_ann*2+1;
   int    ncoeff      = ncoeff_ext * ncoeff_ext * ncoeff_day * ncoeff_ann;
   int    igeo,igsm,iday,iann,md,ma,ic;
   int    n,mgeo,mgsm,i1,i2;
   double harm_a_r,harm_d_r,harm_a_i,harm_d_i;
   
   for (igsm = 0; igsm < ncoeff_ext; igsm++) 
   {
      for (igeo = 0; igeo < ncoeff_ext; igeo++)
      { 
         for (iday = 0;iday <= deg_day; iday++) 
         {
            green_day_r[igsm][igeo][iday] = 0.0;
         }
         for (iann = 0;iann <= deg_ann; iann++) 
         {
            green_ann_r[igsm][igeo][iann] = 0.0;
         }
         for (iday = 0;iday <= deg_day; iday++) 
         {
            green_day_i[igsm][igeo][iday] = 0.0;
         }
         for (iann = 0;iann <= deg_ann; iann++) 
         {
            green_ann_i[igsm][igeo][iann] = 0.0;
         }
      }
   }
   
   ic = 0;
   
   for (n = 1; n <= deg_want; n++)
   {
      for (i1 = 0; i1 <= 2*n; i1++)
      {
         mgsm = i2m(i1);
         igsm = indx(n,mgsm);
         
         for (i2 = 0; i2 <= 2*n; i2++)
         {
            mgeo = i2m(i2);
            igeo = indx(n,mgeo);
            
            for (md = -deg_day; md <= deg_day; md++) 
            {
               if (md == 0)
               {
                  harm_d_r = 1.0;
                  harm_d_i = 0.0;
               }
               if (md > 0) 
               {
                  harm_d_r = SQRT2 * cos(md * WD * fday);
                  harm_d_i = SQRT2 * sin(md * WD * fday);
               }
               if (md < 0)
               {
                  harm_d_r = SQRT2 * sin(md * WD * fday);
                  harm_d_i = SQRT2 * cos(md * WD * fday);
               }
               for (ma = -deg_ann; ma <= deg_ann; ma++) 
               {
                  if (ma == 0) 
                  {
                     harm_a_r = 1.0;
                     harm_a_i = 0.0;
                  }
                  if (ma > 0)
                  {
                     harm_a_r = SQRT2 * cos(ma * WA * fday);
                     harm_a_i = SQRT2 * sin(ma * WA * fday);
                  }
                  if (ma < 0)
                  {
                     harm_a_r = SQRT2 * sin(ma * WA * fday);
                     harm_a_i = SQRT2 * cos(ma * WA * fday);
                  }
                  
                  ic++;
                  
                  if  (md < 0) 
                  {
                     iday = -md;
                  }
                  else
                  { 
                     iday = md; 
                  }
                  
                  green_day_r[igsm][igeo][iday] += harm_d_r * 
                                                   harm_a_r * 
                                                   coeff[ic-1];
                  green_day_i[igsm][igeo][iday] += harm_d_i * 
                                                   harm_a_r * 
                                                   coeff[ic-1];
                  
                  if (ma < 0)
                  {
                     iann = -ma;
                  } 
                  else
                  {
                     iann = ma; 
                  }
                  
                  if (md == 0)/*daily-varying terms average to zero, annually*/
                  {
                     green_ann_r[igsm][igeo][iann] += harm_d_r * 
                                                      harm_a_r * 
                                                      coeff[ic-1];
                     green_ann_i[igsm][igeo][iann] += harm_d_r * 
                                                      harm_a_i * 
                                                      coeff[ic-1];
                     /*                        
                     printf
                        (
                           "%d %d %d %d %d %12.5f\n",
                           igsm,igeo,md,ma,ic,coeff[ic-1]
                        ); 
                     */
                  }
               }/*for ma*/
            }/*for md*/
         }/* for mgeo */
      }/* for mgsm */
   }/*deg_want*/
   
   return(0);
} 
/* gsm2geo_green */
