/*=============================================================================

   POMME: POtsdam Magnetic Model of the Earth
   -------==------=--------=------------=----

   Version 2.3

   2004-Oct-.., Stefan Maus, NOAA.GOV

   2004-Oct-30. add 'pomme_control', prettier code. (rother)
   2004-Nov-08, add 'pomme2d3_extract_deg', modified 'pomme_control'. (rother)
=============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include <pomme3d1.h>

#define  TEST 1
static   FILE *infile;

static int imax
(
   int x,
   int y
)
{ 
   if (x > y)
   { 
      return(x); 
   }
   else
   {
      return(y); 
   }
}

static void readln
(
   char oneline[256]
)
{
   char *temp;

   fgets(oneline, 256, infile);  
   temp = strchr(oneline, '\n');  
   if (temp != NULL) *temp = 0;

   return;
}

static int indx
(
   int n,
   int m
)
{
  int i;

  if (m == 0) 
  {
    i = (n - 1) * (n + 1) + 1;
    return(i-1);
  }
  if (m >= 0)
  {
     i = (n - 1) * (n + 1) + m * 2;
  }
  else
  {
     i = (n - 1) * (n + 1) - m * 2 + 1;
  }
  return(i-1);
} 
/*indx*/

static int pomme_extract_deg
(
   char   *parstring,           /* input string */
   int    *g1,                  /* output first */
   int    *g2                   /* output last  */
)
{
   /* 
   simple, handcrafted reading of selected start degree and end degree from 
   'parstring', two signless integer numbers devided by whitespace,  ',','-' 
   or '/';  tries to be a _bit_  robust, but will not make a real parsing.
   2004-11-08. 
   */
   int  i;
   char *p, *p1, *p2;

   p = parstring;

   /* skip leading whitespace */

   for( ;  strchr(" ,/-", (int)(*p)); p++); if (*p) p1 = p; else return(0);

   /* skip first token */

   for( ; !strchr(" ,/-", (int)(*p)); p++); if (!*p) return(0);

   /* skip delimiting whitespace */

   for( ;  strchr(" ,/-", (int)(*p)); p++); if (*p) p2 = p; else return(0);

   /* try to scan string */

   if (!sscanf(p1,"%d",g1)) return(0);
   if (!sscanf(p2,"%d",g2)) return(0);

   /* ok */
   return(1);      
}

int pomme_control
(
   tmodel *model,
   char   *action
)
{
   /* 
   set and reset some of the flags used inside the 'tmodel' structure by 
   simple command strings; the form 'NO_' set the corresponding entry to zero, 
   the form 'RESET_' reloads the inital value from the default copy.
  
   currently recognized: (right column is default):

                    NO_SM              RESET_SM
                    NO_GSM             RESET_GSM
                    NO_IMFBY           RESET_IMFBY
                    NO_GSM_IND         RESET_GSM_IND
                    NO_GSM_IMFBY_IND   RESET_GSM_IMFBY_IND
                    NO_EST_FAC         RESET_EST_FAC
                    NO_EST             RESET_EST
                    NO_IST             RESET_IST

   and the degree

                    DEGREE #, #
   or
                    DEGREE RESET

   additionally, an internal 'verbose' flag, creating additional output 
   to stdout:

                    VERBOSE_ON         VERBOSE_OFF

   returns:

        0 if all seems to be ok, anything else is an error

   2004-Oct-26. Martin Rother (rother@gfz-potsdam.de).
   */

   int    *c, *d;
   int    f;
   int    ofs, g1, g2, ok;
   double tmp;

   c = model->control;
   d = model->default_control;

   switch (*action)
   {
      case 'D':
         /* 
           test for 'DEGREE ' case only
         */
         if (!strncmp(action,"DEGREE", 6))
         {
            ok = 1;
            switch (*(action+6))
            {
               case '0':      /* unexpected end of string */
                  ok = 0;
                  break;
               case ' ':
                  ofs = 7;
                  break;
               case '-':
                  ofs = 7;
                  break;
               case '=':
                  ofs = 7;
                  break;
               case '_':
                  ofs = 7;
                  break;
               default:
                  ofs = 6;
                  break;
            }
            if (ok)
            {
               /* first test, if a reset command */

               if (!strncmp(action+ofs,"RESET",5))
               {
                  c[POS_FIRST_DEG] = 1;
                  c[POS_LAST_DEG]  = d[POS_0];
                  return(0);
               }

               /* not? now try to extract start/end degree */

               ok = pomme_extract_deg(action+ofs, &g1, &g2);

               if (ok)          /* ok, seems to be an acceptable entry */
               {
                  /* but still needs some limit checks */
                  if (g1 > g2)  /* sort values quietly */
                  {
                     tmp = g1; g1 = g2; g2 = tmp;
                  }
                  if (g1 < 1)
                  {
                     printf
                        (
                           "Min value in %s must be greater or equal one! "\
                           "Reset to 1\n",
                           action
                        );
                     g1 = 1;
                  }
                  if (g2 > c[POS_0])
                  {
                     printf
                        (
                           "Max value in %s exeeds current limit of %d! "\
                           "Reset to default.\n", 
                           action,
                           c[POS_0]
                        );
                     g2 = c[POS_0];
                  }
                  c[POS_FIRST_DEG] = g1;
                  c[POS_LAST_DEG]  = g2;
                  return(0);
               }
            }
         }
         break;
         
      case 'N':
         /* 
           test for 'NO ' case only
         */
         if ((!strncmp(action,"NO ",3))||(!strncmp(action,"NO_",3)))
         {
            if (!strncmp(action+3, "SM",  2))
            {
               f = c[POS_SM]  = 0;                            return(f);
            }
            if (!strncmp(action+3, "GSM", 3))
            {
               f = c[POS_GSM] = 0;                            return(f);
            }
            if (!strncmp(action+3, "IMFBY", 5))
            {
               f = c[POS_IMFBY] = 0;                          return(f);
            }
            if (!strncmp(action+3, "GSM_IND", 7))
            {
               f = c[POS_GSM_IND] = 0;                        return(f);
            }
            if (!strncmp(action+3, "GSM_IMFBY_IND", 13))
            {
               f = c[POS_IMFBY_IND] = 0;                      return(f);
            }
            if (!strncmp(action+3, "EST_FAC", 10))
            {
               f = c[POS_EST_FAC  ] = 0;                      return(f);
            }
            if (!strncmp(action+3, "EST", 3))
            {
               f = c[POS_EST] = 0;                            return(f);
            }
            if (!strncmp(action+3, "IST", 3))
            {
               f = c[POS_IST] = 0;                            return(f);
            }
         }
         break;

      case 'R':
         /* 
           test for 'RESET ' case only
         */
         if ((!strncmp(action,"RESET ",6))||(!strncmp(action,"RESET_",6)))
         {
            if (!strncmp(action+6, "SM", 2))
            {
               f = c[POS_SM] = d[POS_SM];                     return(f);
            }
            if (!strncmp(action+6, "GSM", 3))
            {
               f = c[POS_GSM] = d[POS_GSM];                   return(f);
            }
            if (!strncmp(action+6, "IMFBY", 5))
            {
               f = c[POS_IMFBY] = d[POS_IMFBY];               return(f);
            }
            if (!strncmp(action+6, "GSM_IND", 7))
            {
               f = c[POS_GSM_IND] = d[POS_GSM_IND];           return(f);
            }
            if (!strncmp(action+6, "IMFBY_IND", 9))
            {
               f = c[POS_IMFBY_IND] = d[POS_IMFBY_IND];       return(f);
            }
            if (!strncmp(action+6, "EST_FAC", 7))
            {
               f = c[POS_EST_FAC] = d[POS_EST_FAC];           return(f);
            }
            if (!strncmp(action, "EST", 3))
            {
               f = c[POS_EST] = d[POS_EST];                   return(f);
            }
            if (!strncmp(action, "IST", 3))
            {
               f = c[POS_IST] = d[POS_IST];                   return(f);
            }
         }
         break;

      case 'V':
         /*
           test for 'VERBOSE_' case only
         */
         if 
             (
                (!strncmp(action, "VERBOSE_",8))
                ||
                (!strncmp(action, "VERBOSE ",8))
             )
         {
            if (!strncmp(action+8, "ON", 2))
            {
               f = model->verbose = 1;                          return(f);
            }
            if (!strncmp(action+8, "OFF", 3))
            {
               f = model->verbose = 0;                          return(f);
            }
         }
         break;

      default:
         break;
         
   }
   printf("Illegal command string %s in pomme_control!\n", action);
   return(-1);
}

int pomme_open_file_mp
(
   char     *mainfname,
   char     *sm2geocf,
   char     *gsm2geocf,
   tmodel   *model,
   long     model_nbytes,
   int      verbose
)
{
  char   oneline[256]=" ";
  char   code[7];
  double epoch;
  double g, gdot, gdotdot, gdotdotdot;
  double h, hdot, hdotdot, hdotdotdot;
  int    i, i1, i2;
  int    m, n;
  int    iline, status;
  int    gsm2geo_ncoeff;

  if (model_nbytes < sizeof(tmodel))
  {
     printf
        (
           "Model array needs at least %1d bytes\n",
           sizeof(tmodel)
        );
     exit(31);
  }

  memset(model, 0, sizeof(tmodel)); 
  model->verbose = verbose;

  if (model->verbose) 
  {
     printf
        (
           "reading %s\n", 
           mainfname
        );
  }

  if (infile != NULL)
  {
     infile = freopen(mainfname, "r", infile);
  }
  else
  {
     infile = fopen(mainfname, "r");
  }
  if (infile == NULL) 
  {
     printf
        (
           "file %s could not be opened\n",
           mainfname
        );
     return(38);
  }
  
  next_header:
  
  readln(oneline); 
  if (oneline[0]=='#')
  {
     goto next_header;
  }
  
  sscanf(oneline,"%6s",code);
  if (model->verbose)
  {
     printf
        (
           "next code: %s\n", 
           code
        );
  }
  
  if (strcmp(code, "Epoch:") == 0) 
  {
     fscanf(infile, "%lg%*[^\n]",&epoch);
     getc(infile);
     if (model->verbose)
     {
        printf
           (
              "%s%8.2f\n", 
              oneline, 
              epoch
           );
     }
     model->ref_day = (epoch - 2000.0) * 365.25;
     if ((epoch < 2000) || (epoch > 2010))
     {
        printf
           (
              "unknown epoch %s\n",
              epoch
           ); 
        exit(32);
     }
     goto next_header;
  }

  if (strcmp(code,"IMF-By") == 0) 
  {
     fscanf
        (
           infile, "%lg%lg%*[^\n]",
           &(model->imfby[1]),         /* imfby[0] stays empty */
           &(model->imfby[2])
        );
     getc(infile);
     if (model->verbose)
     {
        printf
           (
              "%s %8.3f %8.3f\n",
              oneline,
              model->imfby[1], 
              model->imfby[2]
           );
     }
     model->control[POS_IMFBY]     = 1;
     model->control[POS_IMFBY_IND] = 1;
     model->deg_imfby              = 1;
     goto next_header;
  }

  if (strcmp(code,"SM-Fac") == 0) 
  {
     fscanf
        (
           infile, "%lg%*[^\n]",
           &(model->est_fac)
        );
     getc(infile);
     if (model->verbose)
     {
        printf
           (
              "%s %8.4f\n",
              oneline, 
              model->est_fac
           );
     }
     model->control[POS_EST_FAC] = 1;
     goto next_header;
  }
  
  if (strcmp(code,"SM-Ext") == 0) 
  {
     fscanf
        (
           infile, 
           "%d%*[^\n]", &(model->deg_sm)
        );
     getc(infile);
     if (model->verbose)
     {
        printf
           (
              "%s degree %d\n",
              oneline, model->deg_sm
           );
     }
     if (model->deg_sm > 0)
     { 
        fscanf
           (
              infile, "%lg%lg%lg%*[^\n]",
              &(model->sm[0]),
              &(model->sm[1]),
              &(model->sm[2])
           ); 
        getc(infile);
        if (model->verbose)
        {
           printf
              (
                 "%10.3f %10.3f %10.3f \n",
                 model->sm[0], 
                 model->sm[1], 
                 model->sm[2]
              ); 
        }
     }
     if (model->deg_sm > 1)
     {
        fscanf
           (
              infile, "%lg%lg%lg%lg%lg%*[^\n]",
              &(model->sm[3]),
              &(model->sm[4]),
              &(model->sm[5]),
              &(model->sm[6]),
              &(model->sm[7])
           );
        getc(infile);
        if (model->verbose)
        {
           printf
              (
                 "%10.3f %10.3f %10.3f %10.3f %10.3f \n",
                 &(model->sm[3]),&(model->sm[4]),&(model->sm[5]),
                 &(model->sm[6]),&(model->sm[7])
              );
        }
     }
     if (model->deg_sm > 2)
     {
        fscanf
           (
              infile, "%lg%lg%lg%lg%lg%lg%lg%*[^\n]",
              &(model->sm[8]),
              &(model->sm[9]),
              &(model->sm[10]),
              &(model->sm[11]),
              &(model->sm[12]),
              &(model->sm[13]),
              &(model->sm[14])
           );
        getc(infile);
        if (model->verbose)
        {
           printf
              (
                 "%10.3f %10.3f %10.3f %10.3f %10.3f %10.3f %10.3f \n",
                 model->sm[8],
                 model->sm[9],
                 model->sm[10],
                 model->sm[11],
                 model->sm[12],
                 model->sm[13],
                 model->sm[14]
              );
        }
     }
     model->control[POS_SM]     = model->deg_sm;
     model->control[POS_SM_IND] = model->deg_sm;
     goto next_header;
  }

  if (strcmp(code,"GSM-Ex") == 0) 
  {
     fscanf
        (
           infile, "%d%*[^\n]", 
           &(model->deg_gsm)
        );
     getc(infile);
     if (model->verbose)
     {
        printf
           (
              "%s degree %d\n",
              oneline, 
              model->deg_gsm
           );
     }
     if (model->deg_gsm > 0)
     { 
        fscanf
           (
              infile, "%lg%lg%lg%*[^\n]",
              &(model->gsm[0]), 
              &(model->gsm[1]), 
              &(model->gsm[2])
           ); 
        getc(infile);
        if (model->verbose)
        {
           printf
              (
                 "%10.3f %10.3f %10.3f \n",
                 model->gsm[0], 
                 model->gsm[1], 
                 model->gsm[2]
              ); 
        }
     }
     if (model->deg_gsm > 1)
     {
        fscanf
           (
              infile, "%lg%lg%lg%lg%lg%*[^\n]",
              &(model->gsm[3]),
              &(model->gsm[4]),
              &(model->gsm[5]),
              &(model->gsm[6]),
              &(model->gsm[7])
           );
        getc(infile);
        if (model->verbose)
        {
           printf
              (
                 "%10.3f %10.3f %10.3f %10.3f %10.3f \n",
                 model->gsm[3], 
                 model->gsm[4], 
                 model->gsm[5],
                 model->gsm[6], 
                 model->gsm[7]
              );
        }
     }
     if ((*model).deg_gsm > 2)
     {
        fscanf
           (
              infile, 
              "%lg%lg%lg%lg%lg%lg%lg%*[^\n]",
              &(model->gsm[8]),
              &(model->gsm[9]),
              &(model->gsm[10]),
              &(model->gsm[11]),
              &(model->gsm[12]),
              &(model->gsm[13]),
              &(model->gsm[14])
           );
        getc(infile);
        if (model->verbose)
        {
           printf
              (
                 "%10.3f %10.3f %10.3f %10.3f %10.3f %10.3f %10.3f \n",
                 model->gsm[8],
                 model->gsm[9],
                 model->gsm[10],
                 model->gsm[11],
                 model->gsm[12],
                 model->gsm[13],
                 model->gsm[14]
              );
        }
     }
     model->control[POS_GSM]     = model->deg_gsm;
     model->control[POS_GSM_IND] = model->deg_gsm;
     goto next_header;
  }

  if (strcmp(code,"Intern") == 0) 
  {
     readln(oneline);/*  Degree: static 1st-deriv 2nd-deriv */
     fscanf
        (
           infile, "%d%d%d%d%*[^\n]",
           &(model->deg_int),
           &(model->deg_1st),
           &(model->deg_2nd),
           &(model->deg_3rd)
        );
     model->control[POS_0] = model->deg_int;
     model->control[POS_1] = model->deg_1st;
     model->control[POS_2] = model->deg_2nd;
     model->control[POS_3] = model->deg_3rd;
     getc(infile);

     if (model->verbose)
     {
        printf
           (
              "%s%6d%6d%6d\n",
              oneline, 
              model->deg_int, 
              model->deg_1st,
              model->deg_2nd, 
              model->deg_3rd
           );
     }

     if (model->deg_int > MAX_0_DEG)
     {
        printf
            (
               "To use this model increase the parameter "\
               "MAX_0_DEG from %3d to %3d \n",
               (int) MAX_0_DEG,
               model->deg_int
            );
        exit(33);
     }
     if (model->deg_1st > MAX_V_DEG)
     {
        printf
            (
               "To use this model increase the parameter "\
               "MAX_V_DEG from %3d to %3d \n",
               (int) MAX_V_DEG,
               model->deg_1st
            );
        exit(34);
     }
     if (model->deg_2nd > MAX_V_DEG)
     {
        printf
            (
               "To use this model increase the parameter "\
               "MAX_V_DEG from %3d to %3d \n",
               (int) MAX_V_DEG,
               model->deg_2nd
            );
        exit(35);
     }
     if (model->deg_3rd > MAX_V_DEG)
     {
        printf
            (
               "To use this model increase the parameter "\
               "MAX_V_DEG from %3d to %3d \n",
               (int) MAX_V_DEG,
               model->deg_3rd);
        exit(36);
     }
     
     readln(oneline);/*  n m glm hlm dglm/dt dhlm/dt ... */
     readln(oneline);/*  ---------------------------     */
      
     for (i = 1; i <= model->deg_int; i++) 
     {
        for (iline = 0; iline <= i; iline++) 
        {
           status = fscanf
              (
                 infile, "%d%d%lg%lg%lg%lg%lg%lg%lg%lg%*[^\n]", 
                 &n, 
                 &m, 
                 &g, 
                 &h, 
                 &gdot, 
                 &hdot, 
                 &gdotdot, 
                 &hdotdot, 
                 &gdotdotdot, 
                 &hdotdotdot
              );
           getc(infile);
           i1 = indx(n, m);
           i2 = indx(n, -m);

           /*
           printf
              (
                 "i, iline, n, m, i1, i2: %d %d %d %d %d\n",
                 i, iline, n, m, i1, i2
              );
           */
           
           model->g[i1] = g;

           if (n <= model->deg_1st)
           {
              model->gs[i1] = gdot;
           } 
           /*
           else
           {
              model->gs[i1] = 0;
           }
           */
           if (n <= model->deg_2nd) 
           {
              model->gss[i1] = gdotdot;
           }
           /*
           else
           {
              model->gss[i1] = 0;
           }
           */           
           if (n <= model->deg_3rd)
           {
              model->gsss[i1] = gdotdotdot;
           } 
           /*
           else
           {
              model->gsss[i1] = 0;
           }
           */           
           if (m > 0) 
           {
              model->g[i2] = h;
              if (n <= model->deg_1st)
              {
                 model->gs[i2] = hdot;
              } 
              /*
              else
              {
                 model->gs[i2] = 0;
              }
              */           
              if (n <= model->deg_2nd)
              {
                 model->gss[i2] = hdotdot;
              }
              /*
              else
              {
                 model->gss[i2] = 0;
              }
              */           
              if (n <= model->deg_3rd)
              {
                 model->gsss[i2] = hdotdotdot;
              }
              /*
              else
              {
                 model->gsss[i2] = 0;
              }
              */           
           }
           /*
           printf
              (
                 "1 n,m g,gs,gss,gsss: "\
                 "%4d %4d %4d %12.3f %12.3f %12.3f %12.3f\n"\
                 "2 n,m g,gs,gss,gsss: "\
                 "%4d %4d %4d %12.3f %12.3f %12.3f %12.3f\n",
                 i1, 
                 n, m,
                 model->g[i1], 
                 model->gs[i1], 
                 model->gss[i1],
                 model->gsss[i1],
                 i2, 
                 n, m,
                 model->g[i2], 
                 model->gs[i2], 
                 model->gss[i2],
                 model->gsss[i2]
              );
           */
        }
     }
     if (infile != NULL)
     {
        fclose(infile);
     }
     infile = NULL;

     /* copy initial state of 'control' to a storage copy */

     for ( i = 0; i < MAXCONTROL; i++)
     {
        model->default_control[i] = model->control[i];
     }

     /*printf("stichprobe gss[0] -> %14.7f\n", model->gss[i]);*/

     if (model->verbose)
     {
        printf
           (
              "last index n, m: %6d, %6d;  g(n,m) = %12.4f g(n,-m) = %12.4f\n",
              n,
              m,
              model->g[indx(n, m)],
              model->g[indx(n, -m)]
           );
     }
  }                               /* if code = internal field  */
  else 
  {
     printf(" Pomme model read error!!!\n");
     exit(39);
  }

  /* prep external */

  model->sm2geo_ncoeff = sm2geo_init
      (
         sm2geocf,
         &(model->sm2geo_deg_ext),
         &(model->sm2geo_deg_day),
         (int) SM2GEO_MAXNCOEFF,
         model->coeff_sm2geo
      );

  model->gsm2geo_ncoeff = gsm2geo_init
      (
         gsm2geocf,
         &(model->gsm2geo_deg_ext),
         &(model->gsm2geo_deg_day),
         &(model->gsm2geo_deg_ann),
         (int) GSM2GEO_MAXNCOEFF,
         model->coeff_gsm2geo
      );

  /* final settings */

  model->control[POS_EST]       = 1;
  model->control[POS_IST]       = 1;

  model->control[POS_FIRST_DEG] = 1;                      /* minimum degree */
  model->control[POS_LAST_DEG]  = model->control[POS_0];  /* maximum degree */

  return(0);
} 
/* pomme_open_file */

int pomme_internal_field_mp 
(
   tmodel *model, 
   double fday, 
   double rad, 
   double theta, 
   double phi,
   double *x, 
   double *y, 
   double *z, 
   double *f
) 
{
   double        costh;
   int           inorm = 1;            /*1=schmid, 2=fully*/
   double        hl, xl, yl, zl, fl;   /* local */
   double        ainc, d, pot;
   double        t, tt, ttt;

   double        g   [MAX_0_NCOEFF];
   double        p   [(MAX_0_DEG+1)*(MAX_0_DEG+2)/2];
   double        dp  [(MAX_0_DEG+1)*(MAX_0_DEG+2)/2];
   double        dx  [MAX_0_DEG*(MAX_0_DEG+2)];
   double        dy  [MAX_0_DEG*(MAX_0_DEG+2)];
   double        dz  [MAX_0_DEG*(MAX_0_DEG+2)];
   double        dpot[MAX_0_DEG*(MAX_0_DEG+2)];

   int           *control;
   register int  i, n, m;

   control = (int*) &(model->control[0]);
   if (model->verbose)
   {
      printf 
         (
            "fday: %12.2f rad: %12.2f theta: %8.2f phi: %8.2f \n",
            fday, rad, theta, phi
         );
   }
   t   = (fday-model->ref_day)/365.25;
   tt  = 0.5 * t * t;
   ttt = tt * t / 3.0;

   if (control[POS_0] > model->deg_int) 
   {
      if (model->verbose)
      {
         printf
             (
                "reset degree(control)= %d to maximal value=%d\n",
                control[POS_0],
                model->deg_int
             );
      }
      control[POS_0] = model->deg_int;
   }

   /* explicit reset of target coefficient array (useful in debug mode) */

   memset(g, 0x0, (size_t)(MAX_0_NCOEFF*sizeof(double)));

   /* fill in coefficient array g */

   for (n = control[POS_FIRST_DEG]; n <= control[POS_LAST_DEG]; n++)
   {
      for (m = -n; m <= n; m++) 
      {
         i    = indx(n, m);
         g[i] = model->g[i];

         /*
         printf
            (
               "-> m, n, i, g gs gss gsss = "\
               "%d %d %d %14.4f %14.4f %14.4f %14.4f\n",
               m, 
               n, 
               i, 
               g[i], 
               model->gs[i], 
               model->gss[i], 
               model->gsss[i]
            );
         */

         if ( (n <= model->deg_1st) && (n <= MAX_V_DEG) )
         {
            g[i] += t   * model->gs[i]; 
         }
         if ( (n <= model->deg_2nd) && (n <= MAX_V_DEG) )
         {
            g[i] += tt  * model->gss[i];
         }
         if ( (n <= model->deg_3rd) && (n <= MAX_V_DEG) )
         {
            g[i] += ttt * model->gsss[i];
         }

         /*
         printf
            (
               "<- m, n, i, g gs gss gsss = "\
               "%d %d %d %14.4f %14.4f %14.4f %14.4f\n",
               m, 
               n, 
               i, 
               g[i], 
               model->gs[i], 
               model->gss[i], 
               model->gsss[i]
            );
         */
      }
   }

   costh = cos(theta);
   plmbar
      (
         &p[0],
         &dp[0],
         &costh,
         &control[POS_LAST_DEG],
         &inorm
      );
   magfdz
      (
         &p[0],  
         &dp[0],
         &theta, 
         &phi, 
         &rad,
         &control[POS_LAST_DEG],
         &g[0],
         &dx[0],
         &dy[0],
         &dz[0],
         &dpot[0],
         &xl,
         &yl,
         &zl,
         &hl,
         &fl,
         &ainc,
         &d,
         &pot
      );
   
   /* copy local to output parameter (not really neccessary) */
   
   *x = xl;
   *y = yl;
   *z = zl;
   *f = fl;
   
   return(0);
} 
/* pomme_internal_field */

int pomme_external_field_mp 
(
   tmodel *model,
   double fday, 
   double rad, 
   double theta, 
   double phi,
   double est, 
   double ist, 
   double imf_by,
   double *x, 
   double *y, 
   double *z
)
{
   int    igeo,igsm,ism,iperiod,gsm2geo_deg;
   double tx,ty,tz,xi,yi,zi,xe,ye,ze,fac;
   double geo_ext_field[MAX_EXT_NCOEFF],geo_ind_field[MAX_EXT_NCOEFF];
   double induc_sm[MAX_EXT_NCOEFF];
   double induc_gsm[MAX_EXT_NCOEFF]; 
   double induc_imfby[MAX_EXT_NCOEFF];
   double est_in_geo[MAX_EXT_NCOEFF]; 
   double ist_in_geo[MAX_EXT_NCOEFF];
   double sm_est_field[MAX_EXT_NCOEFF];
   double sm_ist_field[MAX_EXT_NCOEFF];
   double gsm_imfby_field[MAX_EXT_NCOEFF];
   double sm_in_geo[MAX_EXT_NCOEFF];
   double gsm_in_geo[MAX_EXT_NCOEFF];
   double imfby_in_geo[MAX_EXT_NCOEFF];

   double qfac_day_r[3][6] = 
   {  
       0.3746, 0.3840, 0.3890, 0.3923, 0.3947, 0.3965, 
       0.4112, 0.4287, 0.4382, 0.4444, 0.4489, 0.4524,
       0.3801, 0.4033, 0.4160, 0.4243, 0.4304, 0.4351  /*Utada b*/
   };
   double qfac_day_i[3][6] = 
   {  
       0.0242, 0.0216, 0.0203, 0.0196, 0.0192, 0.0190, 
       0.0444, 0.0402, 0.0382, 0.0370, 0.0365, 0.0362,
       0.0576, 0.0531, 0.0509, 0.0497, 0.0491, 0.0489  /*Utada b*/
   };
   double qfac_ann_r[3][6] = 
   {  
       0.1275, 0.1921, 0.2309, 0.2527, 0.2660, 0.2750,  
       0.0617, 0.1178, 0.1647, 0.1968, 0.2221, 0.2341,
       0.0250, 0.0577, 0.0924, 0.1214, 0.1438, 0.1610 /*Utada b*/
   };
   double qfac_ann_i[3][6] = 
   {  
       0.0894, 0.1093, 0.1024, 0.0924, 0.0841, 0.0778,
       0.0821, 0.1215, 0.1297, 0.1264, 0.1192, 0.1147,
       0.0557, 0.0932, 0.1113, 0.1176, 0.1182, 0.1167 /*Utada b*/
   };

   tgsm2geo_green green_day_r;
   tgsm2geo_green green_ann_r; 
   tgsm2geo_green green_day_i;
   tgsm2geo_green green_ann_i;

   int      ext_deg = MAX_EXT_DEG;
   double   costh,t;
   int      inorm = 1;  /*1=schmid, 2=fully*/
   int      n,mgsm,msm,mgeo;
   int      *control;

   double   h,ainc,d,pot,f;
   double   g[   MAX_EXT_NCOEFF];
   double   p[  (MAX_EXT_DEG+1)*(MAX_EXT_DEG+2)/2];
   double   dp[ (MAX_EXT_DEG+1)*(MAX_EXT_DEG+2)/2];
   double   dx[  MAX_EXT_DEG*(MAX_EXT_DEG+2)];
   double   dy[  MAX_EXT_DEG*(MAX_EXT_DEG+2)];
   double   dz[  MAX_EXT_DEG*(MAX_EXT_DEG+2)];
   double   dpot[MAX_EXT_DEG*(MAX_EXT_DEG+2)];

   control = model->control;

   if (model->verbose)
   {
      printf
         (
            "1: %f %f %f %f %f %f %f\n",
            fday,
            rad,
            theta,
            phi, 
            est, 
            ist, 
            imf_by
         );
   }

   for (igeo=0; igeo < MAX_S_COEFF; igeo++) 
   {
      geo_ext_field[igeo] = 0;
      geo_ind_field[igeo] = 0;
   }

   /* SM */

   if (control[POS_SM] > 0)
   {
      sm2geo
          (
             fday, 
             control[POS_SM], 
             model->sm2geo_deg_ext, 
             model->sm2geo_deg_day,  
             model->coeff_sm2geo,
             model->sm,sm_in_geo
          );

      for (igeo=0; igeo < control[POS_SM]*(control[POS_SM]+2); igeo++)
      {
         geo_ext_field[igeo] +=  sm_in_geo[igeo];
      }
   }

   /* Induced by SM in GEO */

   if (control[POS_SM_IND] > 0)
   {
      sm2geo_green
          (
             fday, 
             control[POS_SM_IND], 
             model->sm2geo_deg_ext, 
             model->sm2geo_deg_day, 
             model->coeff_sm2geo,
             green_day_r, green_day_i
          );
      
      for (igeo=0; igeo < MAX_S_COEFF; igeo++)
      {
         induc_sm[igeo] = 0;
      }
      
      for (n = 1; n <= control[POS_SM_IND]; n++)
      {
         for (msm = -n; msm <= n; msm++)
         {
            ism = indx(n,msm);

            for (mgeo = -n; mgeo <= n; mgeo++)
            {
               igeo = indx(n,mgeo);

               /*green[,,0] is constant term*/

               for (iperiod=1; iperiod <= model->sm2geo_deg_day; iperiod++)
               {
                  induc_sm[igeo] += qfac_day_r[n-1][iperiod - 1] * 
                                    model->sm[ism] * 
                                    green_day_r[ism][igeo][iperiod] 
                                    - 
                                    CONVENTION * qfac_day_i[n-1][iperiod - 1] *
                                    model->sm[ism] * 
                                    green_day_i[ism][igeo][iperiod];
               }/*iperiod*/
            }/*for mgeo*/
         }/*for msm*/
      }/*n*/

      for (igeo = 0; igeo < MAX_S_COEFF; igeo++)
      {
         geo_ind_field[igeo] += induc_sm[igeo];
         /*
         printf
            (
               "geo_ind_field[%d] = %f, delta = %f\n",
               igeo, 
               geo_ind_field[igeo],
               induc_sm[igeo]
            );
         */
      }
   }

   /* EST/IST field */

   if (est != 9999)
   {
      if (control[POS_EST_FAC] > 0) 
      {
         fac = model->est_fac;
      }
      else
      { 
         fac = 1.0;
      }

      if (control[POS_EST] > 0)
      {
         sm_est_field[0] = -fac * est;
         sm_est_field[1] = 0.0;
         sm_est_field[2] = 0.0;

         sm2geo
             (
                fday, 
                1, 
                model->sm2geo_deg_ext, 
                model->sm2geo_deg_day, 
                model->coeff_sm2geo,
                sm_est_field,est_in_geo
             );

         for (igeo=0; igeo < 3; igeo++)
         {
            geo_ext_field[igeo] += est_in_geo[igeo];
         }
      }
      if (control[POS_IST] > 0)
      {
         sm_ist_field[0] = -fac * ist;
         sm_ist_field[1] = 0.0;
         sm_ist_field[2] = 0.0;

         sm2geo
             (
                fday, 
                1, 
                model->sm2geo_deg_ext, 
                model->sm2geo_deg_day, 
                model->coeff_sm2geo,
                sm_ist_field,
                ist_in_geo
             );

         for (igeo=0; igeo < 3; igeo++)
         {
            geo_ind_field[igeo] += ist_in_geo[igeo];
            /*
            printf
               (
                  "geo_ind_field[%d] = %f, delta = %f\n",
                  igeo, 
                  geo_ind_field[igeo],
                  ist_in_geo[igeo]
               );
            */
         }
      }
   }

   /* GSM */

   gsm2geo_deg = imax
       (
          control[POS_GSM_IND],
          control[POS_IMFBY_IND]
       );
   gsm2geo_green
       (
          fday, 
          gsm2geo_deg, 
          model->gsm2geo_deg_ext, 
          model->gsm2geo_deg_day, 
          model->gsm2geo_deg_ann, 
          model->coeff_gsm2geo,
          green_day_r, 
          green_day_i, 
          green_ann_r, 
          green_ann_i
       );

   if (control[POS_GSM] > 0)
   {
      gsm2geo
          (
             fday, 
             control[POS_GSM], 
             model->gsm2geo_deg_ext, 
             model->gsm2geo_deg_day, 
             model->gsm2geo_deg_ann, 
             model->coeff_gsm2geo,
             model->gsm,gsm_in_geo
          );
      for (igeo=0; igeo < control[POS_GSM]*(control[POS_GSM]+2); igeo++)
      {
         geo_ext_field[igeo] += gsm_in_geo[igeo];
         /*
         printf
            (
               "geo_ext_field[%d] = %f, gsm_in_geo = %f\n",
               igeo, 
               geo_ext_field[igeo],
               gsm_in_geo[igeo]
            );
         */
      }   

   }

   /* Induced by GSM in GEO */

   if (control[POS_GSM_IND] > 0)
   {
      for (igeo=0;igeo<MAX_S_COEFF;igeo++) 
      {
         induc_gsm[igeo] = 0;
      }
          
      for (n = 1; n <= control[POS_GSM_IND]; n++)
      {
         for (mgsm = -n; mgsm <=n; mgsm++)
         {
            igsm = indx(n,mgsm);

            for (mgeo = -n; mgeo <=n; mgeo++)
            {
               igeo = indx(n,mgeo);

               /*green[,,0] is constant term*/

               for (iperiod=1; iperiod <= model->gsm2geo_deg_day; iperiod++)
               {
                  induc_gsm[igeo] += qfac_day_r[n-1][iperiod - 1] * 
                                     model->gsm[igsm] * 
                                     green_day_r[igsm][igeo][iperiod]
                                     - 
                                     CONVENTION * qfac_day_i[n-1][iperiod - 1] * 
                                     model->gsm[igsm] * 
                                     green_day_i[igsm][igeo][iperiod];
               }

               /*green[,,0] is constant term*/

               for (iperiod=1; iperiod <= model->gsm2geo_deg_ann; iperiod++)
               {
                  induc_gsm[igeo] += qfac_ann_r[n-1][iperiod - 1] * 
                                     model->gsm[igsm] *       
                                     green_ann_r[igsm][igeo][iperiod]
                                     - 
                                     CONVENTION * qfac_ann_i[n-1][iperiod - 1] * 
                                     model->gsm[igsm] * 
                                     green_ann_i[igsm][igeo][iperiod];
               }
            }/*for mgeo*/
         }/*for mgsm*/
      }/*n*/

      for (igeo=0; igeo < MAX_S_COEFF; igeo++)
      { 
         geo_ind_field[igeo] += induc_gsm[igeo];
         /*
         printf
            (
               "geo_ind_field[%d] = %f, induc_gsm = %f\n",
               igeo, 
               geo_ind_field[igeo],
               induc_gsm[igeo]
            );
         */
      }
   }

   /* IMF-By */

   for (igsm=0; igsm < control[POS_IMFBY]*(control[POS_IMFBY]+2); igsm++)
   {
      gsm_imfby_field[igsm] = imf_by * model->imfby[igsm];
   }

   if (control[POS_IMFBY] > 0)
   {
      gsm2geo
          (
             fday, 
             control[POS_IMFBY], 
             model->gsm2geo_deg_ext, 
             model->gsm2geo_deg_day, 
             model->gsm2geo_deg_ann, 
             model->coeff_gsm2geo,
             gsm_imfby_field,
             imfby_in_geo
          );
      for (igeo=0; igeo < control[POS_IMFBY]*(control[POS_IMFBY]+2); igeo++)
      {
         geo_ext_field[igeo] += imfby_in_geo[igeo];
      }
   }

   /* IMF-By induced */

   if (control[POS_IMFBY_IND] > 0)
   {
      for (igeo=0; igeo < MAX_S_COEFF; igeo++) 
      {
         induc_imfby[igeo] = 0;
      }

      for (n = 1; n <= control[POS_IMFBY_IND]; n++)
      {
         for (mgsm = -n; mgsm <= n; mgsm++)
         {
            igsm = indx(n,mgsm);
            
            for (mgeo = -n; mgeo <= n; mgeo++)
            {
               igeo = indx(n,mgeo);
               
               for (iperiod=1; iperiod <= model->gsm2geo_deg_day; iperiod++) 
               {
                  induc_imfby[igeo] += model->imfby[igsm] * 
                                       imf_by * 
                                       (
                                          qfac_day_r[n-1][iperiod - 1] * 
                                          green_day_r[igsm][igeo][iperiod]
                                          - 
                                          CONVENTION * qfac_day_i[n-1][iperiod - 1] * 
                                          green_day_i[igsm][igeo][iperiod]
                                       );
               }
            }/*for mgeo*/
         }/*for mgsm*/
      }/*n*/
        
      for (igeo=0; igeo < MAX_S_COEFF; igeo++) 
      {
         geo_ind_field[igeo] += induc_imfby[igeo];
         /*
         printf
            (
               "geo_ind_field[%d] = %f, induc_imfby = %f\n",
               igeo, 
               geo_ind_field[igeo],
               induc_imfby[igeo]
            );
         */
      }
   }

   costh = cos(theta);
   
   plmbar
       (
          &p[0],
          &dp[0],
          &costh,
          &ext_deg,
          &inorm
       );
   magfdz
       (
          &p[0],
          &dp[0],
          &theta,
          &phi,
          &rad,
          &ext_deg,
          &geo_ind_field[0],
          &dx[0],
          &dy[0],
          &dz[0],
          &dpot[0],
          &xi,
          &yi,
          &zi,
          &h,
          &f,
          &ainc,
          &d,&pot
       );
   magfdz_ext
       (
          &p[0],
          &dp[0],
          &theta,
          &phi,
          &rad,
          &ext_deg,
          &geo_ext_field[0],
          &dx[0],
          &dy[0],
          &dz[0],
          &dpot[0],
          &xe,
          &ye,
          &ze,
          &h,
          &f,
          &ainc,
          &d,
          &pot
       );

   /*
   printf(" xi, yi, zi = %f, %f, %f\n", xi, yi, zi);
   printf(" xe, ye, ze = %f, %f, %f\n", xe, ye, ze);
   */

   *x = xe + xi;
   *y = ye + yi;
   *z = ze + zi;
   
   if (model->verbose && TEST) 
   {
      magfdz_ext
          (
             &p[0],&dp[0],
             &theta,&phi,&rad,
             &ext_deg,&sm_in_geo[0],
             &dx[0],&dy[0],&dz[0],&dpot[0],
             &tx,&ty,&tz,&h,&f,&ainc,&d,&pot
          );
      printf("SM extern:  %10.4f %10.4f %10.4f\n",tx,ty,tz);
      
      magfdz
          (
             &p[0],&dp[0],
             &theta,&phi,&rad,
             &ext_deg,&induc_sm[0],
             &dx[0],&dy[0],&dz[0],&dpot[0],
             &tx,&ty,&tz,&h,&f,&ainc,&d,&pot
          );
      printf("SM induced: %10.4f %10.4f %10.4f\n",tx,ty,tz);
      
      magfdz_ext
          (
             &p[0],&dp[0],
             &theta,&phi,&rad,
             &ext_deg,&est_in_geo[0],
             &dx[0],&dy[0],&dz[0],&dpot[0],
             &tx,&ty,&tz,&h,&f,&ainc,&d,&pot
          );
      printf("SM Est:     %10.4f %10.4f %10.4f\n",tx,ty,tz);
      
      magfdz
          (
             &p[0],&dp[0],
             &theta,&phi,&rad,
             &ext_deg,&ist_in_geo[0],
             &dx[0],&dy[0],&dz[0],&dpot[0],
             &tx,&ty,&tz,&h,&f,&ainc,&d,&pot
          );
      printf("SM Ist:     %10.4f %10.4f %10.4f\n",tx,ty,tz);
      
      magfdz_ext
          (
             &p[0],&dp[0],
             &theta,&phi,&rad,
             &ext_deg,&gsm_in_geo[0],
             &dx[0],&dy[0],&dz[0],&dpot[0],
             &tx,&ty,&tz,&h,&f,&ainc,&d,&pot
          );
      printf("GSM extern: %10.4f %10.4f %10.4f\n",tx,ty,tz);
      
      magfdz
          (
             &p[0],&dp[0],
             &theta,&phi,&rad,
             &ext_deg,&induc_gsm[0],
             &dx[0],&dy[0],&dz[0],&dpot[0],
             &tx,&ty,&tz,&h,&f,&ainc,&d,&pot
          );
      printf("GSM ind:    %10.4f %10.4f %10.4f\n",tx,ty,tz);
      
      magfdz_ext
          (
             &p[0],&dp[0],
             &theta,&phi,&rad,
             &ext_deg,&imfby_in_geo[0],
             &dx[0],&dy[0],&dz[0],&dpot[0],
             &tx,&ty,&tz,&h,&f,&ainc,&d,&pot
          );
      printf("GSM IMFBy:  %10.4f %10.4f %10.4f\n",tx,ty,tz);
      
      magfdz
          (
             &p[0],&dp[0],
             &theta,&phi,&rad,
             &ext_deg,&induc_imfby[0],
             &dx[0],&dy[0],&dz[0],&dpot[0],
             &tx,&ty,&tz,&h,&f,&ainc,&d,&pot
          );
      printf("IMFBy ind:  %10.4f %10.4f %10.4f\n",tx,ty,tz);
   }
   
   return(0);
} 
/* pomme_external_field */
   
int pomme_field_mp 
(
   tmodel *model,
   double fday, 
   double rad, 
   double theta, 
   double phi,
   double est, 
   double ist, 
   double imf_by,
   double *x, 
   double *y, 
   double *z, 
   double *f
)
{
   double dx,dy,dz;

   pomme_internal_field_mp 
      (
         model, fday, rad, theta, phi, x, y, z, f
      );
   
   pomme_external_field_mp 
      (
         model, fday, rad, theta, phi, est, ist, imf_by, &dx, &dy, &dz
      );
   
   *x += dx;
   *y += dy;
   *z += dz;
   *f  = sqrt(*x**x + *y**y + *z**z);
   
   return(0);
} 
/* pomme_field_mp */
