                                
/* Max relative rms error is <0.5% for deg-1 and <2% for deg-2 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <x2geo.h>

static FILE *cfile;

static int indx
(
   int n,
   int m
)
{
  int i;

  if (m == 0) 
  {
     i = (n - 1) * (n + 1) + 1;
     return(i-1);
  }
  if (m >= 0)
  {
     i = (n - 1) * (n + 1) + m * 2;
  }
  else
  {
     i = (n - 1) * (n + 1) - m * 2 + 1;
  }
  return(i-1);
}  
/*indx*/

static int i2m
(
   int i
)
{
   int m;

   m = (i+1) / 2;
   if ((i & 1) == 0)
   {
      m = -m;
   }
  return(m);
}  
/*i2m*/

int sm2geo_init
(
   char   fname[],
   int    *deg_ext,
   int    *deg_day,
   int    maxcoeff,
   double coeff[]
)
{
   int i,ism,igeo,iday,ncoeff;
   int n,mgeo,msm,i1,i2;
   int ncoeff_ext,ncoeff_day,ie0,ie1,id,in0,im0,im1;

   cfile = fopen(fname, "r");
   if (cfile == NULL) 
   {
      printf
          (
             "file %s could not be opened\n",fname
          ); 
      exit(21);
   }

   fscanf(cfile,"%*[^\n]");
   getc(cfile);
   fscanf(cfile,"%d%d%*[^\n]",deg_ext,deg_day);

   ncoeff_ext = *deg_ext*(*deg_ext+2);
   ncoeff_day = *deg_day*2+1;
   ncoeff     = ncoeff_ext * ncoeff_ext * ncoeff_day;

   if (ncoeff > maxcoeff) 
   {
      printf
          (
             "array size for coefficients is too small"
          ); 
      exit(22);
   }

   getc(cfile);
   fscanf(cfile,"%*[^\n]");
   getc(cfile);
   fscanf(cfile,"%*[^\n]");
   getc(cfile);

   i = 0;

   for (n = 1; n <= *deg_ext; n++)
   {
      for (i1 = 0; i1 <= 2*n; i1++)
      {
         msm = i2m(i1);
         ism = indx(n,msm);

         for (i2 = 0; i2 <= 2*n; i2++)
         {
            mgeo = i2m(i2);
            igeo = indx(n,mgeo);

            for (iday = 0; iday < ncoeff_day; iday++)
            {
               fscanf
                   (
                      cfile,
                      "%d%d%d%d",
                      &in0,
                      &im0,
                      &im1,
                      &id
                   );
               /*
               printf
                  (
                     "sm2geo read: n %d m %d m %d iday %d\n",
                     in0,im0,im1,id
                  ); 
               */
               ie0 = indx(in0,im0);
               ie1 = indx(in0,im1);

               if 
                   (
                      (ie0 != ism)          || 
                      (ie1 != igeo)         ||
                      (id  != iday-*deg_day) 
                   )
               {
                  printf
                      (
                         "loop: n %d m %d m %d iday %d\n",
                         n,msm,mgeo,id
                      );
                  printf
                      (
                         "read: n %d m %d m %d iday %d\n",
                         in0,im0,im1,id
                      );
                  printf
                      (
                         "sm2geo: something is wrong with input file! %s\n",
                         "%d <> %d or %d <> %d or %d <> %d",
                         ie0,ism,ie1,igeo,id,iday-*deg_day
                      );
                  exit(23);
               }

               fscanf
                   (
                      cfile,"%lg", &coeff[i]
                   );
               /*                 
               printf
                  (
                     "%12.5f\n",
                     coeff[i]
                  ); 
               */

               i++;

               fscanf(cfile,"%*[^\n]");
               getc(cfile);
               
            }/* for iday */
         }/* for mgeo */
      }/* for msm */
   }/*deg_ext*/

   fclose(cfile);
   return(ncoeff);
}

int sm2geo
(
   double fday,                     /* chosen */
   int    deg_want,          
   int    deg_ext,/* given */
   int    deg_day,
   double coeff[], 
   double smcoeff[],
   double geocoeff[]
)
{
   int ncoeff_want = deg_want*(deg_want+2);
   int ncoeff_ext  = deg_ext*(deg_ext+2);
   int ncoeff_day  = deg_day*2+1;
   int ncoeff      = ncoeff_ext * ncoeff_ext * ncoeff_day;
   int igeo,ism,iday,md,ma,ic;
   int n,mgeo,msm,i1,i2;
   double harm_a,harm_d;

   for (igeo = 0; igeo < ncoeff_ext; igeo++) 
   {
      geocoeff[igeo] = 0.0;
   }

   ic = 0;

   for (n = 1; n <= deg_want; n++)
   {
      for (i1 = 0; i1 <= 2*n; i1++)
      {
         msm = i2m(i1);
         ism = indx(n,msm);

         for (i2 = 0; i2 <= 2*n; i2++)
         {
            mgeo = i2m(i2);
            igeo = indx(n,mgeo);

            for (md = -deg_day; md <= deg_day; md++) 
            {
               if (md == 0) harm_d = 1.0;
               if (md > 0)  harm_d = SQRT2 * cos(md * WD * fday);
               if (md < 0)  harm_d = SQRT2 * sin(md * WD * fday);
               geocoeff[igeo]     += smcoeff[ism] * harm_d * coeff[ic];

               ic++;
               /*                    
               printf
                  (
                     "%d %d %d %d %d %12.5f\n",
                     ism,igeo,md,ic,coeff[ic]
                  ); 
               */
            }/*for md*/
         }/* for mgeo */
      }/* for msm */
   }/*deg_want*/

  return(0);
} 
/*sm2geo*/

int sm2geo_green
(
   double        fday,                        /* chosen */
   int           deg_want,         
   int           deg_ext,                     /* given */
   int           deg_day,
   double        coeff[], 
   tsm2geo_green green_day_r,
   tsm2geo_green green_day_i
)
{
   int ncoeff_want = deg_want*(deg_want+2);
   int ncoeff_ext  = deg_ext*(deg_ext+2);
   int ncoeff_day  = deg_day*2+1;
   int ncoeff      = ncoeff_ext * ncoeff_ext * ncoeff_day;

   int igeo,ism,iday,md,ic;
   int n,mgeo,msm,i1,i2;
   double harm_d_r,harm_d_i;

   for (ism = 0; ism < ncoeff_ext; ism++)
   {
      for (igeo = 0; igeo < ncoeff_ext; igeo++)
      { 
         for (iday = 0;iday <= deg_day;iday++) 
         {
            green_day_r[ism][igeo][iday] = 0.0;
         }
         for (iday = 0;iday <= deg_day;iday++) 
         {
            green_day_i[ism][igeo][iday] = 0.0;
         }
      }
   }
    
   ic = 0;

   for (n = 1; n <= deg_want; n++)
   {
      for (i1 = 0; i1 <= 2*n; i1++)
      {
         msm = i2m(i1);
         ism = indx(n,msm);

         for (i2 = 0; i2<=2*n; i2++)
         {
            mgeo = i2m(i2);
            igeo = indx(n,mgeo);

            for (md = -deg_day; md <= deg_day; md++) 
            {
               if (md == 0)
               {
                  harm_d_r = 1.0;
                  harm_d_i = 0.0;
               }
               if (md > 0) 
               {
                  harm_d_r = SQRT2 * cos(md * WD * fday);
                  harm_d_i = SQRT2 * sin(md * WD * fday);
               }
               if (md < 0)
               {
                  harm_d_r = SQRT2 * sin(md * WD * fday);
                  harm_d_i = SQRT2 * cos(md * WD * fday);
               }
               ic++;

               if  (md < 0)
               {
                  iday = -md;
               } 
               else
               {
                  iday = md; 
               }
               green_day_r[ism][igeo][iday] += harm_d_r * coeff[ic-1];
               green_day_i[ism][igeo][iday] += harm_d_i * coeff[ic-1];
               /*                        
               printf
                  (
                     "%d %d %d %d %d %12.5f\n",
                     ism,igeo,md,ic,coeff[ic-1]
                  ); 
               */
            }/*for md*/
         }/* for mgeo */
      }/* for msm */
   }/*deg_want*/

  return(0);
} 
/* sm2geo_green */
