/*=============================================================================

   POMME: POtsdam Magnetic Model of the Earth
   -------==------=--------=------------=----
   Version 2.3

   2004-Oct, Stefan Maus, NOAA.

=============================================================================*/

#include <x2geo.h>

/* 
         if external field degree > 3, code needs complete rewrite:
*/
#define R_ERDE               6371.2
#define CONVENTION           (1.0)   /* 1.0 for exp(iwt), -1.0 for exp(-iwt) */

#define GEO2GSE              1
#define GSE2GEO              2
#define GEO2GSM              3
#define GSM2GEO              4
#define GEO2SM               5
#define SM2GEO               6

#define POS_0                0       /* static internal */
#define POS_1                1       /* SV */
#define POS_2                2       /* sec acc */
#define POS_3                3       /* 3rd temp deriv */
#define POS_SM               4       /* static ring current */
#define POS_SM_IND           5       /* if asymmetric then could induce field, 
                                        not implemented 
                                     */
#define POS_EST              6       /* external RC field */
#define POS_IST              7       /* induced RC field */
#define POS_GSM              8       /* external GSM static field */
#define POS_GSM_IND          9       /* induced internal*/
#define POS_IMFBY            10      /* external IMFBY correlated field */
#define POS_IMFBY_IND        11      /* induced internal*/
#define POS_EST_FAC          12      /* factor for Est/Ist field */

#define POS_FIRST_DEG        13      /* selected first */
#define POS_LAST_DEG         14      /* selected last  */

#define MAXCONTROL           15

typedef                      int tcontrol[MAXCONTROL];

#define MAX_0_DEG            120
#define MAX_V_DEG            20
#define MAX_EXT_DEG          3

#define MAX_0_NCOEFF         (MAX_0_DEG   * (MAX_0_DEG   + 2))
#define MAX_V_NCOEFF         (MAX_V_DEG   * (MAX_V_DEG   + 2))
#define MAX_EXT_NCOEFF       (MAX_EXT_DEG * (MAX_EXT_DEG + 2))  

/*
          main coefficient structure
*/

typedef struct tmodel
{
    tcontrol                  control;
    tcontrol                  default_control;
    int                       verbose; 

    int                       deg_imfby;
    int                       deg_sm;
    int                       deg_gsm;
    int                       deg_int;
    int                       deg_1st;
    int                       deg_2nd;
    int                       deg_3rd;
   
    double                    ref_day;
    double                    imfby[3]; 
    double                    est_fac;
    
    double                    sm   [MAX_EXT_NCOEFF];
    double                    gsm  [MAX_EXT_NCOEFF];
    double                    g    [MAX_0_NCOEFF];
    double                    gs   [MAX_V_NCOEFF];
    double                    gss  [MAX_V_NCOEFF];
    double                    gsss [MAX_V_NCOEFF];
    
    int                       sm2geo_deg_ext;
    int                       sm2geo_deg_day;
    tsm2geo_coeff             coeff_sm2geo;
    int                       sm2geo_ncoeff;

    int                       gsm2geo_deg_ext;
    int                       gsm2geo_deg_day;
    int                       gsm2geo_deg_ann;
    tgsm2geo_coeff            coeff_gsm2geo;
    int                       gsm2geo_ncoeff;
} 
tmodel;

/*
------- tmodel.h -------
*/
