      subroutine plmbar(p,dp,z,lmax,inorm)

      implicit real*8 (a-h,o-z)
c
c  evaluates normalized associated legendre function p(l,m) as function of
c   z=cos(colatitude) using recurrence relation starting with p(l,l) 
c   and then increasing l keeping m fixed.  normalization is: 
c   integral(y(l,m)*y(l,m))=4.*pi, where y(l,m) = p(l,m)*exp(i*m*longitude),
c   which is incorporated into the recurrence relation. p(k) contains p(l,m)
c   with k=(l+1)*l/2+m+1; i.e. m increments through range 0 to l before 
c   incrementing l. routine is stable in single and real*8 precision to
c   l,m = 511 at least; timing proportional to lmax**2
c   r.j.o'connell 7 sept. 1989

c   a.jackson 19 october 1989  code added at end:
c   (1) choice of normalisation added ---
c       if inorm = 1 schmidt normalisation is chosen, where
c       p[schmidt](l,m) = 1/sqrt(2*l+1)*p[normalised](l,m)
c       inorm = 2 for fully normalised harmonics, otherwise error
c   (2) derivatives added and stored in dp(k)
c       using same arrangement as for p(k)
c
      dimension p(*),dp(*)
c     --dimension of p, dp must be (lmax+1)*(lmax+2)/2 in calling program
      if (lmax.lt.0.or.abs(z).gt.1.d0) pause 'plmbar.f: bad arguments'
      if(inorm.ne.1.and.inorm.ne.2)then
        write(6,99)
99      format('plmbar.f: inorm incorrect: use inorm=1 for schmidt norm'
     >  ,/,'                                   inorm=2 for full norm.')
        stop
      endif
        
c       --case for p(l,0) 
      pm2=1.d0
      p(1)=1.d0
      dp(1)=0.d0
      if (lmax .eq. 0) return
      pm1=z
      p(2)=dsqrt(3.d0)*pm1
      k=2
      do 4 l=2,lmax
         k=k+l
         plm=(dfloat(2*l-1)*z*pm1-dfloat(l-1)*pm2)/dfloat(l)
         p(k)=dsqrt(dfloat(2*l+1))*plm
         pm2=pm1
 4       pm1=plm
c     --case for m > 0
         pmm = 1.d0
         sintsq = (1.d0-z)*(1.d0+z)
         fnum = -1.0d0
         fden = 0.0d0
         kstart = 1
   do 20 m =1 ,lmax
c     --case for p(m,m) 
      kstart = kstart+m+1
      fnum = fnum+2.0d0
      fden = fden+2.0d0
      pmm = pmm*sintsq*fnum/fden
      pm2 = dsqrt(dfloat(4*m+2)*pmm)
     p(kstart) = pm2
      if (m .eq. lmax) goto 100
c     --case for p(m+1,m)
      pm1=z*dsqrt(dfloat(2*m+3))*pm2
      k = kstart+m+1
      p(k) = pm1
c     --case for p(l,m) with l > m+1
     if (m .lt. (lmax-1)) then
        do 10 l = m+2,lmax
           k = k+l
           f1=dsqrt(dfloat((2*l+1)*(2*l-1))/dfloat((l+m)*(l-m)))
           f2=dsqrt(dfloat((2*l+1)*(l-m-1)*(l+m-1))
     &          /dfloat((2*l-3)*(l+m)*(l-m)))
           plm=z*f1*pm1-f2*pm2
       p(k) = plm
       pm2 = pm1
 10    pm1 = plm
c      write(6,*) 'plmbar 1: ', pm1, pm2
      endif
 20   continue

 100  continue
      
c     choice of normalisation:
      if(inorm.eq.1)then
         k=1
        do 30 l=1,lmax
           fac=1.d0/dsqrt(dfloat(2*l+1))
           do 30 m=0,l
              k=k+1
              p(k)=p(k)*fac
 30        continue
        endif

c     now find derivatives of p(z) wrt theta, where z=cos(theta)
c     recurrence is same regardless of normalisation, since l=constant
        dp(2)=-p(3)
        dp(3)=p(2)
        k=3
        do 200 l=2,lmax
           
           k=k+1
c     treat m=0 and m=l separately
           dp(k)=-dsqrt(dfloat(l*(l+1))/2.d0)*p(k+1)
           dp(k+l)=dsqrt(dfloat(l)/2.d0)*p(k+l-1)
           do 300 m=1,l-1
              k=k+1
              fac1=dsqrt( dfloat( (l-m)*(l+m+1) ) )
              fac2=dsqrt( dfloat( (l+m)*(l-m+1) ) )
              if(m.eq.1)fac2=fac2*dsqrt(2.d0)
              dp(k)=0.5d0*( fac2*p(k-1) - fac1*p(k+1) )

c             write(6,*) 'dp(k) 2: ', k, dp(k)
300       continue
          k=k+1
          
 200   continue
       return
       end
