
FUNCTION pomme_calc, model,          $;in: input model (hidden struct)
                     vlen,           $;in: long length of array
                     fday,           $;in: date/time as MJD(2000.0)
                     r,              $;in: position, radius in km
                     theta,          $;in: position, theta (colat) in deg
                     phi,            $;in: position, phi in deg
                     e_st,           $;in: parameter est
                     i_st,           $;in: parameter ist
                     by_imf,         $;in: parameter b(y) imf
                     lib   = lib,    $;explicit lib, but a default should work.
                     trace = trace

;+=============================================================================
;
;  IDL wrapper for POMME code of Stefan Maus.
;
;  Input:
;
;  Keywords:
;
;  Returns:
;
;                 structure
;
;  2004-Nov-11. Martin Rother (rother@gfz-potsdam.de).
;  2006-Jan-27. pomme-3: simplified by using a system-var !pommelib as default 
;-=============================================================================

   ;---------------------------------------------------------------------------
   ;   CHECK
   ;---------------------------------------------------------------------------

   IF n_params() NE 9 THEN BEGIN 
      ;
      stop, 'illegal usage!'
      ;
   ENDIF 

   IF NOT istrue(model) THEN BEGIN 
      ;
      stop, 'insigificant model parameter structure'
      ;
   ENDIF

   ;---------------------------------------------------------------------------
   ;   CREATE
   ;---------------------------------------------------------------------------

   bm     = dblarr(4L,vlen)
   be     = dblarr(3L,vlen)

   ;---------------------------------------------------------------------------
   ;  CALL POMME CALC
   ;---------------------------------------------------------------------------

   s = call_external(                                 $
                      keyword_set(lib)?lib:!pommelib, $ ;sysvar !pommelib req.!
                      'pomme_calc',                   $
                      model,                          $
                      vlen,                           $
                      fday,                           $
                      r,    theta, phi,               $
                      e_st, i_st,  by_imf,            $
                      bm,                             $
                      be                              $
                    )
   return,          {                                 $
                      status : s,                     $
                      bm     : bm,                    $
                      be     : be,                    $
                      b      : bm[0:2,*]+be           $
                    }
END 
;==============================================================================
;  EOF pomme_calc.pro
;==============================================================================
