
FUNCTION pomme_control, model,       $;in: pomme hidden model structure
                        cmd,         $;in: string command
                     lib   = lib      ;explicit lib, but a default should work.
;+=============================================================================
;
;  IDL wrapper for POMME2 code of Stefan Maus.
;
;  Input:
;
;  Keywords:
;
;     lib         string         library
;
;  Returns:
;
;                 long           a pointer to model structure (hidden),
;                                used in 'pomme_calc' and 'pomme_control'.
;                                do not modify the return value; this should
;                                work in unix-like environments.
;
;
;  but, I really don't know how this will work in Windows; all depends on the
;  lifetime of memory allocated from the C code inside the interface; to avoid
;  memory leaks the 'model'-pointer should be destroyed by 'pomme_kill'.
;
;  2004-Nov-11. Martin Rother (rother@gfz-potsdam.de).
;  2006-Jan-27. simplified with system-var !pommelib for pomme-3.
;-=============================================================================

   ;---------------------------------------------------------------------------
   ;   CHECK
   ;---------------------------------------------------------------------------

   IF n_params() NE 2 THEN BEGIN 
      ;
      stop, 'usage: status = pomme_control(model, command)'
      ;
   ENDIF 

   IF NOT istrue(model) THEN BEGIN 
      ;
      stop, 'insigificant model parameter structure'
      ;
   ENDIF

   IF size(model, /type) NE 3 THEN stop, 'illegal first parameter (model)'
   IF size(cmd,   /type) NE 7 THEN stop, 'illegal command string parameter'

   this = long(model)

   ;---------------------------------------------------------------------------
   ;  CALL POMME CONTROL
   ;---------------------------------------------------------------------------

   len  = n_elements(cmd)
   stat = lonarr(len)

   FOR i = 0L, len-1L DO BEGIN 
      ;
      stat[i] = call_external(keyword_set(lib)?lib:!pommelib,   $
                              'pomme_cntl',                     $
                              this,                             $
                              cmd[i]                            )
      ;
   ENDFOR

   return, stat
END 
;==============================================================================
;  EOF pomme_control.pro
;==============================================================================
