
FUNCTION pomme_init, pomme_cf,       $;in: POMME coeff file/path
                     sm2geo_cf,      $;in: SM    parameter file/path
                     gsm2geo_cf,     $;in: GSM   parameter file/path
                     verbose,        $
                     lib   = lib      ;explicit lib, but a default should work.
;+=============================================================================
;
;  IDL wrapper for POMME2 code of Stefan Maus.
;
;  Input:
;
;     pomme_cf    string         path/file to POMME2  coeff. file
;     sm2geo_cf   string         path/file to SM2GEO  parameter file
;     gsm2geo_cf  string         path/file to GSM2GEO parameter file
;     verbose     integer        logical flag: be verbose (1) or not (0)
;
;  Keywords:
;
;     trace       flag           write trace-messages to stdout
;
;  Returns:
;
;                 long           a pointer to model structure (hidden),
;                                used in 'pomme_calc' and 'pomme_control'.
;                                do not modify the return value; this should
;                                work in unix-like environments.
;
;
;  but, I really don't know how this will work in Windows; all depends on the
;  lifetime of memory allocated from the C code inside the interface; to avoid
;  memory leaks the 'model'-pointer should be destroyed by 'pomme_kill'.
;
;  2004-Nov-11. Martin Rother (rother@gfz-potsdam.de).
;  2006-Jan-27. simplified with system-var !pommelib for pomme-3.
;-=============================================================================

   ;---------------------------------------------------------------------------
   ;   CHECK
   ;---------------------------------------------------------------------------

   IF n_params() NE 4 THEN BEGIN 
      ;
      stop, 'usage: model = pomme_init(cfpomme, cfsm2geo, cfgsm2geo, verbose)'
      ;
   ENDIF 

   IF size(pomme_cf,   /type) NE 7 THEN stop, 'illegal first parameter'
   IF size(sm2geo_cf,  /type) NE 7 THEN stop, 'illegal second parameter'
   IF size(gsm2geo_cf, /type) NE 7 THEN stop, 'illegal third parameter'

   use_verbose = long(istrue(verbose)) ;force to be a long (is 'int' in C)

   ;---------------------------------------------------------------------------
   ;  CALL POMME INIT
   ;---------------------------------------------------------------------------

   return, call_external(keyword_set(lib)?lib:!pommelib,        $
                         'pomme_init',                          $
                         pomme_cf,                              $
                         sm2geo_cf,                             $
                         gsm2geo_cf,                            $
                         use_verbose                            )
END 
;==============================================================================
;  EOF pomme_init.pro
;==============================================================================
