;;=============================================================================
;;  
;;  GFZ Section 2.3 internal: read combined est/ist and omni data table
;;                            assembled for internal use (leaded by MJD2000.0
;;                            time information) and read/prepare supplemental 
;;                            data needed by geo-magnetic field model POMME 2.3
;;                            code.
;;
;;  History:
;;
;;  2004-Oct-27. Martin Rother (rother@gfz-potsdam.de)
;;  2005-Feb-17. something was odd. bit modified.
;;=============================================================================

FUNCTION istimf_prep, parmjd, trace = t

  ;============================================================================
  ;  
  ;  returns a structure 'pompar' with By_IMF (GSM) from selfmade data 
  ;  'omni2'-all-file and the 'est/ist'-file table contributed by 
  ;  stefan maus, the author of the 'POMME 2.3' model.
  ;
  ;  input:                     
  ;
  ;     parmjd            structure  with items:
  ;
  ;                                  estist string     
  ;
  ;                                  path/filename of 'est/ist' table
  ;                                  contributed by stefan maus
  ; 
  ;                                  omni2 string     
  ;
  ;                                  path/filename of assembled 'omni2'-file
  ;                                  with records leaded by 'mjd-2000.0' time 
  ;                                  instead of the original date/time columns.
  ;                              
  ;  output:
  ;
  ;                       struct     with items
  ;
  ;                                  pompar struct     
  ;
  ;                                  assembled information with 'by_imf (GSM)'
  ;                                  and the est/ist indices.
  ;
  ;                                  ifo    struct     
  ;
  ;                                  bunch of additional 
  ;                                  information, for debug purposes only
  ;
  ;  2004-Oct-27. Martin Rother (rother@gfz-potsdam.de)
  ;  2005-Feb-17. something was odd. bit modified.
  ;===========================================================================
   
   ;--------------------------------------------------------------------------
   ;  CUSTOMIZE HERE
   ;--------------------------------------------------------------------------

   defaults      = { estist : '/export/db/Indices/Dst/Est_Ist_index.lis',  $
                     omni2  : '/export/db/Indices/omni/omni2_XXXX.list'    }
   
   omni2_usecol  = 46           ;
   byimf_column  = 13           ; two less then in the description
   byimf_maxval  = 998.0        ;
   estist_usecol = 3            ;

   ;---------------------------------------------------------------------------
   ;  CHECKS
   ;---------------------------------------------------------------------------

   use = defaults & mergetags, use, parmjd
   
   IF NOT file_test(use.estist,/read) THEN               $
      die, 'illegal estist-file '+q(use.estist)
   IF NOT file_test(use.omni2, /read) THEN               $
      die, 'illegal selected estist-file '+q(use.omni2)

   ;---------------------------------------------------------------------------
   !,'READ FILES',t
   ;---------------------------------------------------------------------------

   o = rlst(use.omni2,  trace = trace)
   e = rlst(use.estist, trace = trace)
  
   ;---------------------------------------------------------------------------
   !,'EXTRACT',t
   ;---------------------------------------------------------------------------
  
   by_imf = o.d([0,byimf_column],*) 
   n      = where(by_imf[1,*] LT byimf_maxval, cnt) 
   IF cnt GT 0 THEN BEGIN 
      ;
      by_imf = by_imf(*,n)
      ;
   ENDIF ELSE BEGIN 
      ;
      by_imf = !nan
      ;
   ENDELSE
  
   ;---------------------------------------------------------------------------
   !,'CREATE STRUCTURE',t
   ;---------------------------------------------------------------------------

   return, {                                                 $
             by_imf : by_imf,                                $ 
             estist : e.d,                                   $
             avg    : {                                      $
                        est    : !zero,                      $
                        ist    : !zero,                      $
                        by_imf : !zero                       $
                      }                                      $
           }
END
;;=============================================================================
;;EOF istimf_prep.pro
;;=============================================================================
