/*  
    Magnetic field model 'Pomme'

    Control Routine

    C Core:          Stefan Maus,   October 16, 2004
    Matlab wrapper:  Martin Rother, October 17, 2004, version 2.3

   January 27, 2006: version for 'pomme-3' without significant modifications
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <mex.h>
#include <pomme3d1.h>

/*-----------------------------------------------------------------------------

   (matlab) mex wrapper

   2004.10.16.
   2006.Jan.27. no significant modification during tests for 'pomme-3'.
-----------------------------------------------------------------------------*/

void mexFunction
(
   int nlhs,         mxArray *plhs[],
   int nrhs,   const mxArray *prhs[]     
)
{
   tmodel  *pmodel;
   int     n,m, slen, stat;
   char    *cmd;
   char    *s;                  /* pointer to string */
      
   /* 
     Examine input (right-hand-side) arguments,
     which are the model-structure and a single command string
   */

   if (nrhs != 2) 
   {
      mexErrMsgTxt("Two input parameters required.");
   }

   if (mxIsDouble(prhs[0]) != 1)
   {
      mexErrMsgTxt("First parameter must be of type double.");
   }
   if (mxIsChar(prhs[1]) != 1)
   {
      mexErrMsgTxt("Second parameter must be of type string.");
   }
   
   /*
      get input string pointer
   */

   slen = (mxGetM(prhs[1]) * mxGetN(prhs[1])) + 1;
   cmd  = mxCalloc(slen, sizeof(char));
   stat = mxGetString(prhs[1], cmd, slen+1);
   if (stat != 0)
   {
      mexErrMsgTxt("Could not convert command string data.");
   }

   /* 
      explicit long pointer to the start of the continous memory
   */

   pmodel = (tmodel*) mxGetPr(prhs[0]);    

   /* call toplevel subroutine */

   pomme_control
      (
         pmodel,                /* pointer into sufficient memory */
         cmd                    /* some silly action              */
      );
   
   return;                      /* that's all folks, no tests, no return val */

}

/* EOF pomme_cntr.c */
