/*  
    Magnetic field model 'Pomme' 

    Initialization 

    C/Fortran Core:  Stefan Maus,   October 16, 2004
    Matlab wrapper:  Martin Rother, October 17, 2004, version 2.3

    January 27, 2006: version for 'pomme-3' without significant modifications
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <mex.h>
#include <pomme3d1.h>

#define  VERBOSE        0       /* NO DEBUG HERE */

/*-----------------------------------------------------------------------------

   (matlab) mex wrapper

   2004.10.16.
-----------------------------------------------------------------------------*/

void mexFunction
(
   int nlhs,         mxArray *plhs[],
   int nrhs,   const mxArray *prhs[]     
)
{
   tmodel  *pmodel;
   int     n,m;

   char    pomme23_cf[256];
   char    sm2geo_cf[256];
   char    gsm2geo_cf[256];
      
   /* 
     Examine input (right-hand-side) arguments,
     which are only the names for the coeff. files.
   */

   if (nrhs != 3) 
   {
      mexErrMsgTxt("Three input parameters required.");
   }

   if (mxIsChar(prhs[0]) != 1)
   {
      mexErrMsgTxt("First parameter must be of type string.");
   }
   if (mxIsChar(prhs[1]) != 1)
   {
      mexErrMsgTxt("Second parameter must be of type string.");
   }
   if (mxIsChar(prhs[2]) != 1)
   {
      mexErrMsgTxt("Third parameter must be of type string.");
   }
   
   /*
      get input string pointer
   */

   if (mxGetString(prhs[0], pomme23_cf, 255) != 0)
   {
      mexErrMsgTxt("Could not convert POMME coeff string data.");
   }
   if (mxGetString(prhs[1],sm2geo_cf, 255) != 0)
   {
      mexErrMsgTxt("Could not convert SM->GEO parfile string data.");
   }
   if (mxGetString(prhs[2], gsm2geo_cf, 255) != 0)
   {
      mexErrMsgTxt("Could not convert GSM->GEO parfiel string data.");
   }

   /* 
      create 'left hand' output parameter (model struct)
   */

   n       = sizeof(tmodel);
   m       = (sizeof(char)*n) / sizeof(double) + 1;

   plhs[0] = mxCreateDoubleMatrix(1, m, mxREAL);

   if (!plhs[0])
   {
      mexErrMsgTxt("Not enough memory available for model structure");
   }

   /* 
      explicit long pointer to the start of the continous memory
   */

   pmodel  = (tmodel*) mxGetPr(plhs[0]);    

   /* call toplevel subroutine */

   pomme_open_file_mp
      (
         pomme23_cf,            /* main pomme 2.3 coefficient file */
         sm2geo_cf,             /* parameter file name and path */
         gsm2geo_cf,            /* parameter file name and path */
         pmodel,                /* pointer into sufficient memory */
         n,                     /* redundant */
         VERBOSE                /* no debug here */
      );

   return;                      /* that's all folks */

}

/* EOF pomme_init.c */
