%%=============================================================================
%%  
%%  GFZ Section 2.3 internal: interpolate combined est/ist and omni data table
%%                            assembled for internal use (leaded by MJD2000.0
%%                            time information) and read/prepare supplemental 
%%                            data needed by geo-magnetic field model POMME 2.3
%%                            code.
%%
%%  History:
%%
%%  2004-Oct-27. Martin Rother (rother@gfz-potsdam.de)
%%=============================================================================

function this = istimf_calc               ...
      (                                   ...
          pompar,                         ...
          mjd                             ...
      )
  %============================================================================
  %  
  %  returns a structure with 'interpolation'-results using structure 'pompar' 
  %  with By_IMF (GSM) from selfmade data 'omni2'-all-file and the 
  %  'est/ist'-file table contributed by stefan maus, the author of the 
  %  'POMME 2.3' model and the 'mjd' double array with the dates/times this
  %  values are desired.
  %
  %  the implicit loop should be used, because an explicit loop may be SLOW!
  %
  %  input:                     
  %
  %     varname           dim/type   comment 
  %     -----------------------------------------------------------------------
  %     pompar            struct     structure red by 'istimf_prep.m'
  % 
  %     mjd               double(n)  single value or array of 'mjd.fraction'
  %                                  values.
  %                              
  %  output:
  %
  %     this              struct     result of the interpolation, 
  %                                  items have the length of 'mjd'.
  %
  %  2004-Oct-27. Martin Rother (rother@gfz-potsdam.de)
  %===========================================================================

  %---------------------------------------------------------------------------
  %  USUALLY AN IMPLICIT LOOP!
  %---------------------------------------------------------------------------
  
  est    = interp1(pompar.estist(1,:), pompar.estist(3,:), mjd);
  ist    = interp1(pompar.estist(1,:), pompar.estist(4,:), mjd);
  by_imf = interp1(pompar.by_imf(1,:), pompar.by_imf(2,:), mjd);
  
  %---------------------------------------------------------------------------
  %  CREATE STRUCTURE
  %---------------------------------------------------------------------------
 
  this.mjd    = mjd;
  this.est    = est;
  this.ist    = ist;
  this.byimf  = by_imf;
  
%%EOF istimf_calc.m
