%%=============================================================================
%%  
%%  GFZ Section 2.3 internal: read combined est/ist and omni data table
%%                            assembled for internal use (leaded by MJD2000.0
%%                            time information) and read/prepare supplemental 
%%                            data needed by geo-magnetic field model POMME 2.3
%%                            code.
%%
%%  History:
%%
%%  2004-Oct-27. Martin Rother (rother@gfz-potsdam.de)
%%=============================================================================

function [pompar, ifo] = istimf_prep        ...
      (                                     ...
          estist_mjdfile,                   ...
          omni2_mjdfile                     ...
      )
  %============================================================================
  %  
  %  returns a structure 'pompar' with By_IMF (GSM) from selfmade data 
  %  'omni2'-all-file and the 'est/ist'-file table contributed by 
  %  stefan maus, the author of the 'POMME 2.3' model.
  %
  %  input:                     
  %
  %     varname           dim/type   comment 
  %     -----------------------------------------------------------------------
  %     estist_mjdfile    string     path/filename of 'est/ist' table
  %                                  contributed by stefan maus
  % 
  %     omni2_mjdfile     string     path/filename of assembled 'omni2'-file
  %                                  with records leaded by 'mjd-2000.0' time 
  %                                  instead of the original date/time columns.
  %                              
  %  output:
  %
  %     pompar            struct     assembled information with 'by_imf (GSM)'
  %                                  and the est/ist indices.
  %     ifo               struct     optional, bunch of additional 
  %                                  information, for debug purposes only
  %
  %  2004-Oct-27. Martin Rother (rother@gfz-potsdam.de)
  %===========================================================================

  %---------------------------------------------------------------------------
  %  CUSTOMIZE HERE
  %---------------------------------------------------------------------------
  
  omni2_usecol  = 47;
  byimf_column  = 14;                   % two less then in the description
  byimf_maxval  = 998.0;
  estist_usecol = 4;
  
  %---------------------------------------------------------------------------
  %  READ OMNI FILE
  %---------------------------------------------------------------------------
  
  h         = rd_n_col('o',    omni2_mjdfile);
  [o,oinfo] = rd_n_col('r', h, omni2_usecol);
  rd_n_col('c',h);
  
  %---------------------------------------------------------------------------
  %  EXTRACT
  %---------------------------------------------------------------------------
  
  by_imf = o([1,byimf_column],:);             clear o;
  n      = find(by_imf(2,:) < byimf_maxval);
  byimf  = by_imf(:,n);                       clear by_imf;
  
  %---------------------------------------------------------------------------
  %  READ EST/IST FILE
  %---------------------------------------------------------------------------

  h         = rd_n_col('o',    estist_mjdfile);
  [e,einfo] = rd_n_col('r', h, estist_usecol);
  rd_n_col('c',h);
  
  %---------------------------------------------------------------------------
  %  CREATE STRUCTURE
  %---------------------------------------------------------------------------

  pompar.by_imf = byimf;
  pompar.estist = e;
  
  ifo.oinfo     = oinfo;
  ifo.einfo     = einfo;

%%EOF istimf_prep.m
