%==============================================================================
%
%   testing: 'run3.m' 
%
%   output to stdout, using input arrays
%   and manipulate calculation by some 'pomme_control' commands
%
%   2004-Nov-10. Martin Rother (rother@gfz-potsdam.de).
%   2006-Feb-02. for Pomme 3.0/3.1.
%==============================================================================

%%  CUSTOMIZE HERE: 

%%           where the parameter files are...

pomme_cf   = '../D/pomme-3.1.cof';
sm2geo_cf  = '../D/sm2geo_coeff';
gsm2geo_cf = '../D/gsm2geo_coeff';

%%           commands:

commands   = char(                                                         ...
                  'NO GSM',                                                ...
                  'NO SM',                                                 ...
                  'DEGREE 20, 60',                                         ...
                  'RESET_GSM',                                             ...
                  'RESET_SM',                                              ...
                  'DEGREE RESET',                                          ...
                  'DEGREE 20, 200'                                         ...
                 );

%%           output format to stdout

format     = ['   Input: %d %8.3f %8.3f %8.3f %7.3f %6.2f %6.2f %4.1f\n\n',...
              'Internal: %9.3f %9.3f %9.3f %9.3f\n',                       ...
              'External: %9.3f %9.3f %9.3f\n'                                ];

%==============================================================================

%%  CREATE INPUT ARRAYS

for i = 1:10

  fday(i)    = 300.1 + i*33.07;
  r(i)       = 6371.2 + i*50;
  theta(i)   = (3.0 + i*15.2) * pi / 180.0; 
  phi(i)     = 22*i  * pi / 180.0;          

  imf_by(i)  = -5.0  + i;
  e_st(i)    = -10.0 * i;
  i_st(i)    = 0.3   * e_st(i);
  
end

%%  CALL INITIALISATION (LOAD CF AND PAR)

model      = pomme_init(pomme_cf, sm2geo_cf, gsm2geo_cf);

%%  COMMAND LOOP

cmdnr      = size(commands);
cmdnr      = cmdnr(1);

for i = 1:cmdnr

   disp(sprintf('---------- Command: %s\n', commands(i,:)));
   
   %%  THIS COMMAND
   pomme_control(model, commands(i,:));
   
   %%  CALL WITH ARRAY
   [bm, be]   = pomme_calc(model, fday, r, theta, phi, e_st, i_st, imf_by);

   %%  DISPLAY RESULT ARRAY
   for i = 1:10
  
     disp(sprintf(format,                                              ...
                  i,                                                   ...
                  fday(i),                                             ...
                  r(i), theta(i), phi(i),                              ...
                  imf_by(i), e_st(i), i_st(i),                         ...
                  bm(:,i)',                                            ...
                  be(:,i)'                                              ));
   end
  
end

%==============================================================================
