#!/usr/local/bin/perl -w -- # -*-Perl-*-

$INFO = 
   (
   "repomme [util]                   (--EXP--)  2005-Nov-16 (GFZ,Martin Rother)

    reformat/modify cofficient set of pomme model, writes to stdout.

    Using:

      repomme [-nostatic][-nofirst][-nosecond] cfinfile

         -nostatic        set first h and g columns to zero
         -nofirst         set hd and gd columns to zero
         -nosecond        set hdd and gdd columns to zero

   Remarks:

      ++ MINOR, EXPERIMENTAL
      ++ No POD documentation.

   Requirements:

      -- Cwd              (standard lib)
      -- Sys::Hostname    (standard lib)

   1000 Bugs. Ok, on a good day some less. 
   (GFZ Potsdam, rother\@gfz-potsdam.de)
   \n"
   );

#------------------------------------------------------------------------------
#  MODIFICATIONS
#------------------------------------------------------------------------------
#
#  2005-Nov-16. 
#------------------------------------------------------------------------------

use Cwd;                        # standard library
use Sys::Hostname;              # standard library

#==============================================================================
#                            CUSTOMIZATION
#==============================================================================

%C = 
(
 nostatic   => 0,
 nofirst    => 0,
 nosecond   => 0,
 cfinfile   => '',
 cformat1   => '%3d',
 cformat2   => '%4d  ',
 vformat1   => '%11.4f ',
 vformat2   => '%8.4f '
);

#------------------------------------------------------------------------------
#  TINY UTILS:
#------------------------------------------------------------------------------

sub P{print "$Key $_[0]\n";}                             # print to stdout
sub T{print "$Key $_[0]\n" if ($C{trace});}              # trace (test only)
sub D{die "$Err $_[0]\n";}                               # stop
sub U{do {} until (!unlink(@_));}

sub Now
{
   my($now);
   $now                    = time;                       # system-second
   ($s,$min,$h,$d,$mon,$y) = localtime($now);            # extracted
   return POSIX::strftime("%Y-%m-%d %H:%M:%S",$s,$min,$h,$d,$mon,$y);
}

#------------------------------------------------------------------------------
#  PREPARATION UTILITIES
#------------------------------------------------------------------------------

sub Process_Information
{                               # grab system information from operating sys
   #
   # global: $now,       number, second 
   #         $s1time,    string, time string as 'YYYY-MM-DD hh:mm:ss'
   #         $s2time,    string, time string as 'YYYY-MM-DD hh-mm-ss' 
   #         $today      string, date-string as 'YYYY-MM-DD'
   #         $user       string, actual user
   #         $host       string, actual host
   #         $wdir       string, acutal working directory

   $C{host} = hostname();       # hostname
   $C{user} = getlogin() || (getpwuid("$<"))[0] || &D("illegal user!!");
   $C{wdir} = cwd();            # actual working directory
}

sub Local_Info
{  
   # extract local information

   @tmp  = split(/\n/, $INFO);
   @tmp  = split(" ", ($Id = $tmp[0]));
   $Key  = "$tmp[0]";
   $Err  = "#!$Key-Error:";
   $Key .= ":";
   &Process_Information;
}

sub Scan_Environment
{                               # check some environment variables
}

#------------------------------------------------------------------------------
#  INIT/DEINIT
#------------------------------------------------------------------------------

sub Init
{
   my($f);
   &Local_Info;                 # extract some internal information
   &Scan_Environment;           # scan some environment variables first

   foreach (@ARGV)              # command line arguments
   {
      /^-v$/          && do {die "$Id\n";             last;};
      /^-h$/          && do {die $INFO;               last;};
      /^-t$/          && do {$C{trace}     = 1;       next;};
      /^-test$/       && do {$C{test}      = 1;       next;};
      /^-nostatic$/   && do {$C{nostatic}  = 1;       next;};
      /^-nosecond$/   && do {$C{nosecond}  = 1;       next;};
      /^-nofirst$/    && do {$C{nofirst}   = 1;       next;};
      /^(.*)$/        && do 
      {
         $f = $1;
       F:
         {
            (!$C{cfinfile}) && do 
            {
               if ( -f $f )
               {
                  $C{cfinfile} = $f;
               }
               else
               {
                  &D("can't find cf infile >$f<");
               };
               last;
            };
            &D("Too much options! Use -h for help.");
         };
         next;
      }
   }
   &D("infile not given!")   unless (   $C{cfinfile});
   &D("illegal input file!") unless (-f $C{cfinfile});
}

sub Deinit
{
}

#------------------------------------------------------------------------------
#   READ
#------------------------------------------------------------------------------

sub Read_Cf_Infile
{
   my($r,@T);

   open(R,$C{cfinfile}) || &D("can't open input file $C{cfinfile}");

   $head   = 1;                 # head flag
   $C{cnt} = 0;                 # record counter

   while (<R>)
   {
      @T =split(' ',$_);        # tokens of record
      if ($head)
      {
       H:
         {
            /^(\s*n\s+m\s+.*)\s*$/ && do 
            {
               $C{columns} = $1;         # column string
               $C{columnt} = [@T];       # column tokens array
               $C{columnn} = scalar @T;  # number of column tokens
               $head       = 0;
               last;
            };
            /^\s*([^:]+):(.*)\s*$/ && do
            {
               $key = $1;
               if ($2)
               {
                  $C{H}{$key}{sameline} = $2;
                  &T("sameline: $2");
               }
               else
               {
                  $C{H}{$key}{sameline} = "";
               };
               push(@{$C{H}{order}},$key);
               next;
            };
            /^(.+)\s*$/            && do
            {
               if ($key)
               {
                  push(@{$C{H}{$key}{nextlines}},$1);
               }
               next;
            };
            /^\s*$/                && do
            {
               if ($key)
               {
                  push(@{$C{H}{$key}{nextlines}},"");
               }
               next;
            };
            &D("unexpected line: $_");
         }
      }
      else
      {
       B:
         {
            /^(\s*---+\s*)$/ && do
            {
               $C{line} = $1;
               next;
            };
            /^\s*(\d+)\s+(\d+)\s+/ && do
            {
               # data entry
               if (scalar(@T) == $C{columnn})
               {
                  push(@{$C{body}{$C{cnt}}},@T);
                  $C{cnt}++;
               }
               else
               {
                  &T("irregular body entry: $_");
               }
               next;
            };
            &T("unexpected record: $_");
         }
      }
   }
}

sub Write_Cf_Stdout
{
   my($k,$r,$c,$i);

   foreach $k (@{$C{H}{order}})
   {
      print "$k: ";
      if (exists($C{H}{$k}{sameline}))
      {
         print "$C{H}{$k}{sameline}\n";
      }
      else
      {
         print "\n";
      }
      if (exists($C{H}{$k}{nextlines}))
      {
         foreach $r (@{$C{H}{$k}{nextlines}})
         {
            print "$r\n";
         }

      }
   }
   print "$C{columns}\n";
   print "$C{line}\n";

   for($i = 0; $i < $C{cnt}; $i++)
   {
      printf($C{cformat1}, $C{body}{$i}[0]);
      printf($C{cformat2}, $C{body}{$i}[1]);      

      if ($C{nostatic})
      {
         printf($C{vformat1}, 0.0);
         printf($C{vformat1}, 0.0);
      }
      else
      {
         printf($C{vformat1}, $C{body}{$i}[2]);
         printf($C{vformat1}, $C{body}{$i}[3]);
      }

      if ($C{columnn} >= 6)
      {
         if ($C{nofirst})
         {
            printf($C{vformat2}, 0.0);
            printf($C{vformat2}, 0.0);
         }
         else
         {
            printf($C{vformat2}, $C{body}{$i}[4]);
            printf($C{vformat2}, $C{body}{$i}[5]);
         }
      }
      if ($C{columnn} >= 8)
      {
         if ($C{nosecond})
         {
            printf($C{vformat2}, 0.0);
            printf($C{vformat2}, 0.0);
         }
         else
         {
            printf($C{vformat2}, $C{body}{$i}[6]);
            printf($C{vformat2}, $C{body}{$i}[7]);
         }
      }

      for( $c = 8; $c < $C{columnn}; $c++)
      {
         printf($C{vformat2}, $C{body}{$i}[$c]);
      }
      print "\n";
   }
}

#------------------------------------------------------------------------------
#  RUN FRAME
#------------------------------------------------------------------------------

sub Run
{
   &Read_Cf_Infile;
   &Write_Cf_Stdout;
}

#==============================================================================
#
#  MAIN
#
#==============================================================================

sub Main
{ 
   &Init; 
   &Run;
   &Deinit; 
} 

&Main; 

#==============================================================================
#  EOF repomme.pl
#==============================================================================
