/*=============================================================================

   Example program (with redundant loops for performance tests...) for:

   POMME: POtsdam Magnetic Model of the Earth
   -------==------=--------=------------=----

   Version 3.1

   2004-Oct-xx, Stefan Maus, NOAA.GOV
   2004-Oct-xx. minor modifications, changed intendation (rother).
   2004-Nov-09. modifications driven by stefans rewrite of core routines.
   2006-Jan-24. for Version 3.1, only some name entries changed yet.
=============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include <pomme3d1.h>

/* CUSTOMIZE: */

#define MAINFNAME       "D/pomme-3.1.cof"
#define SM2GEOCF        "D/sm2geo_coeff"
#define GSM2GEOCF       "D/gsm2geo_coeff"
#define VERBOSE         0

static  tmodel          model;

/* MAIN BODY */

int main()
{
   long i, l;
   double E_st, I_st, fday, theta, phi, f, r, rrel, lat, lon, rad, alt;
   double az, w, x, y, z, mx, my, mz, ex, ey, ez, IMF_By;
   char *action;

   /*
   printf
       (
          "opening %s\n", 
          MAINFNAME
       );
   */
   pomme_open_file_mp
       (
          MAINFNAME, 
          SM2GEOCF, 
          GSM2GEOCF, 
          &model, 
          sizeof(tmodel),
          VERBOSE
       );

   pomme_control(&model, "DEGREE 1 14");

   for (l = 1; l <= 1000; l++)
   {
      for (i = 1; i <= 10; i++)
      {
         /* create test input */
         
         fday    = 300.1 + i*33.07;
         r       = 6371.2 + i*50;
         theta   = (3.0 + i*15.2) * M_PI / 180.0;    /*colat in radian*/
         phi     = 22*i  * M_PI / 180.0;             /*lon in radian*/
         
         IMF_By  = -5.0  + i;
         E_st    = -10.0 * i;
         I_st    = 0.3   * E_st;
         
         /*
         printf
            (
               "<- %8.2f %10.2f %9.4f %9.4f   %8.2f %8.2f %8.2f\n",
               fday,
               r,
               theta * 180.0/M_PI,
               phi   * 180.0/M_PI,
               E_st,
               I_st,
               IMF_By
            );
         */

         pomme_internal_field_mp
            (
               &model, 
               fday, 
               r, 
               theta, 
               phi, 
               &mx, 
               &my, 
               &mz, 
               &f
            );
         pomme_external_field_mp
            (
               &model, 
               fday, 
               r, 
               theta, 
               phi, 
               E_st, 
               I_st,
               IMF_By, 
               &ex, 
               &ey, 
               &ez
            );
         printf
            (
               "-> %10.2f %10.2f %10.2f %10.2f  %10.2f%10.2f%10.2f\n\n", 
               mx, my, mz, f, ex, ey, ez
            );
      }
   }
}
/* EOF example3.c */
